﻿using UnityEngine;

namespace Mod
{
    public class ClawBehaviour : CanShoot
    {
        private bool gripping = false;

        FixedJoint2D joint;

        float fullMass = 0f;

        private Vector2 barrelPosition = new Vector2(0.1f, 0f);
        private Vector2 barrelDirection = new Vector2(1f, 0f);

        public void OnGripped(GripBehaviour g)
        {
            GetComponent<SpriteRenderer>().sortingOrder = g.gameObject.GetComponent<SpriteRenderer>().sortingOrder + 1;
            GetComponent<SpriteRenderer>().sortingLayerID = g.gameObject.GetComponent<SpriteRenderer>().sortingLayerID;
        }

        public override void Shoot()
        {
            RaycastHit2D ray = Physics2D.Raycast(BarrelPosition, BarrelDirection, 0.15f);

            if (gripping == false && ray.collider != null && ray.collider.gameObject.layer == 9)
            {
                joint = this.gameObject.AddComponent<FixedJoint2D>();
                joint.enableCollision = false;
                joint.breakForce = Mathf.Infinity;
                joint.breakTorque = Mathf.Infinity;
                joint.anchor = barrelPosition;

                joint.connectedBody = ray.collider.gameObject.GetComponent<Rigidbody2D>();

                joint.connectedAnchor = ray.transform.InverseTransformPoint(ray.point);

                gripping = true;
            }
            else
            {

                var allPhysicalChildren = ray.transform.root.GetComponentsInChildren<PhysicalBehaviour>();
                if (allPhysicalChildren.Length == 0) return;

                for (int i = 0; i < allPhysicalChildren.Length; i++)
                {
                    fullMass += allPhysicalChildren[i].gameObject.GetComponent<Rigidbody2D>().mass;
                }

                joint.connectedBody.AddForce(BarrelDirection * 0f * fullMass, ForceMode2D.Impulse);
                gameObject.GetComponent<Rigidbody2D>().AddForce(BarrelDirection * (Mathf.Clamp(fullMass, 0f, 0f)), ForceMode2D.Impulse);
                Destroy(joint);
                gripping = false;
                fullMass = 0f;
            }
        }
        void Use()
        {
            Shoot();
        }

        void OnCollisionEnter2D(Collision2D other)
        {
            if (!gripping && gameObject.GetComponent<Rigidbody2D>().velocity.magnitude >= 5f)
            {
                var allPhysicalChildren = other.transform.root.GetComponentsInChildren<PhysicalBehaviour>();
                if (allPhysicalChildren.Length == 0) return;

                for (int i = 0; i < allPhysicalChildren.Length; i++)
                {
                    fullMass += allPhysicalChildren[i].gameObject.GetComponent<Rigidbody2D>().mass;
                }

                other.gameObject.GetComponent<Rigidbody2D>().AddForce(gameObject.GetComponent<Rigidbody2D>().velocity * Mathf.Clamp(other.gameObject.GetComponent<Rigidbody2D>().mass, 0f, 25f), ForceMode2D.Impulse);

                fullMass = 0f;
            }
        }

        public override Vector2 BarrelPosition { get { return transform.TransformPoint(barrelPosition); } }
        public Vector2 BarrelDirection { get { return transform.TransformDirection(barrelDirection) * transform.localScale.x; } }
    }
}