using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

namespace Mod
{
    public class SunPower : CanShoot
    {

        private GameObject Fusion;
        private PhysicalBehaviour physicalBehaviour;
        private GameObject FusionGlow;
        private Vector2 barrelPosition = new Vector2(2.9f, 0f);
        private Vector2 barrelDirection = new Vector2(1, 0);
        public bool upgraded = false;
        private LineRenderer laser;
        private LayerMask layers;
        private LayerMask bounds;
        private GameObject components;

        private RaycastHit2D[] victimBuffer = new RaycastHit2D[128];

        public void Start()
        {

            Fusion = gameObject;
            physicalBehaviour = GetComponent<PhysicalBehaviour>();

            layers = LayerMask.GetMask("Objects", "CollidingDebris", "Bounds");
            bounds = LayerMask.GetMask("Bounds");

            components = new GameObject("components", typeof(Optout));
            components.transform.SetParent(transform);
            components.transform.localPosition = new Vector3(3.5f, 0f);
            laser = components.AddComponent<LineRenderer>();
            laser.SetPositions(new[] { Vector3.zero, Vector3.right * 1000 });
            laser.useWorldSpace = false;
            laser.sharedMaterial = ModAPI.FindMaterial("VeryBright");
            laser.sortingLayerName = "Background";
            laser.startColor = new Color(0.78f, 0f, 1f, 1);
            laser.endColor = new Color(0.78f, 0f, 1f, 1);
            laser.widthMultiplier = 1f;
            laser.enabled = false;

            FusionGlow = new GameObject();
            FusionGlow.transform.parent = Fusion.transform;
            FusionGlow.transform.localPosition = new Vector3(3.5f, 0f);
            FusionGlow.transform.localScale = gameObject.transform.parent.localScale;
            FusionGlow.transform.localRotation = Quaternion.identity;
            var FusionSprite = FusionGlow.AddComponent<SpriteRenderer>();

            FusionSprite.sprite = ModAPI.LoadSprite("th.png");
            FusionSprite.material = ModAPI.FindMaterial("VeryBright");
            FusionSprite.color = new Color(0.78f, 0f, 1f, 1);
            FusionSprite.sortingLayerName = "Top";

        }
        private void Use()
        {
            Shoot();
        }

        public override void Shoot()
        {

            StartCoroutine(Fire());
        }

        private IEnumerator Fire()
        {
            const int iterations = 100;
            const float duration = 0.01f;

            const float interval = duration / iterations;

            laser.startColor = new Color(0.78f, 0f, 1f, 1);
            laser.endColor = new Color(0.78f, 0f, 1f, 1);
            laser.widthMultiplier = 1f;

            laser.enabled = true;
            for (int i = 0; i < iterations; i++)
            {
                var end = Physics2D.Raycast(BarrelPosition, BarrelDirection, 10000, bounds);
                var endPos = end.transform == null ? (Vector3)BarrelPosition + (Vector3)BarrelDirection * 10000 : (Vector3)end.point;
                var victimCount = Physics2D.LinecastNonAlloc(BarrelPosition, endPos, victimBuffer, layers);
                laser.SetPosition(1, components.transform.InverseTransformPoint(endPos));

                for (int n = 0; n < victimCount; n++)
                {
                    var victim = victimBuffer[n];
                    ExplosionCreator.CreatePulseExplosion(victim.point, 25f, 15, true);
                    victim.transform.SendMessage("Slice", SendMessageOptions.DontRequireReceiver);
                    victim.transform.SendMessage("Break", SendMessageOptions.DontRequireReceiver);
                    victim.transform.SendMessage("Crush", SendMessageOptions.DontRequireReceiver);
                }

                CameraShakeBehaviour.main.Shake(1, transform.position);
                physicalBehaviour.rigidbody.AddRelativeForce(new Vector2(0f * transform.localScale.normalized.x, 0), ForceMode2D.Impulse);
                yield return new WaitForSeconds(interval);
            }
            laser.enabled = false;
            yield return new WaitForSeconds(5f);
        }

        public override Vector2 BarrelPosition { get { return transform.TransformPoint(barrelPosition); } }

        public Vector2 BarrelDirection { get { return transform.TransformDirection(barrelDirection) * transform.localScale.x; } }

    }
}
