using System;
using System.Collections.Generic;
using System.IO;
using UnityEngine;
using UnityEngine.Events;

namespace Mod
{
    public class Mod
    {
        public static void Main()
        {
        CategoryBuilder.Create("OF - Ostrumians", "Adds Ostrumians",ModAPI.LoadSprite("Category1.png"));
        CategoryBuilder.Create("OF - Technology", "Adds machinery",ModAPI.LoadSprite("Category2.png"));
        CategoryBuilder.Create("OF - Tools", "Adds weapons and tools",ModAPI.LoadSprite("Category3.png"));

            //Ostrumian Cargo Crate
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "Ostrumian Cargo Crate -OF",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "A cargo crate. That's it.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/CargoCrateView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/CargoCrate.png");
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Bottom";
                    }
                }
            );
            //Ostrumian Shipping Container
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "Ostrumian Shipping Container -OF",
                    NameToOrderByOverride = "Aa",
                    DescriptionOverride = "A large shipping container",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/ShippingContainerView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ShippingContainer.png");

                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Bottom";

                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 45f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 45f;
                        Instance.GetComponent<PhysicalBehaviour>().rigidbody.mass = 45f;

                        Instance.FixColliders();
                    }
                }
            );
            //Map Block
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "Map Block -OF",
                    NameToOrderByOverride = "ZZZZZZZZ",
                    DescriptionOverride = "A block for basic terrain making.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/MapBlockView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/MapBlock.png");
                        Instance.FixColliders();

                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";

                        Instance.GetOrAddComponent<NoTemperature>();

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Soft");
                        phys.SimulateTemperature = false;
                        phys.Disintegratable = false;
                        phys.Properties.Flammability = 0f;
                    }
                }
            );
            //Infragium Chunk
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Infragium Chunk -OF",
                    NameToOrderByOverride = "ZZZ",
                    DescriptionOverride = "A small sized metal chunk mined from an outcrop of Infragium.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/InfragiumChunkView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/InfragiumChunk.png");
                        Instance.FixColliders();

                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";

                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.5f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.5f;
                        Instance.GetComponent<PhysicalBehaviour>().rigidbody.mass = 0.5f;

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Metal");
                        phys.Properties.Flammability = 0f;
                    }
                }
            );
            //Infragium Node
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "Infragium Node -OF",
                    NameToOrderByOverride = "ZZZZ",
                    DescriptionOverride = "A medium sized metal outcrop of Infragium. It is insanely tough, and requires specialised machinery to mine.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/InfragiumNodeView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/InfragiumNode.png");
                        Instance.FixColliders();

                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Bulletproof");
                        phys.Properties.Flammability = 0f;
                    }
                }
            );
            //Obstructonium Chunk
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Obstructonium Chunk -OF",
                    NameToOrderByOverride = "ZZZZZ",
                    DescriptionOverride = "A small sized crystal chunk mined from an outcrop of Obstructonium.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/ObstructoniumChunkView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ObstructoniumChunk.png");
                        Instance.FixColliders();

                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";

                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.5f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.5f;
                        Instance.GetComponent<PhysicalBehaviour>().rigidbody.mass = 0.5f;

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Glass");
                        phys.Properties.Flammability = 0f;
                    }
                }
            );
            //Obstructonium Node
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Mirror"),
                    NameOverride = "Obstructonium Node -OF",
                    NameToOrderByOverride = "ZZZZZZ",
                    DescriptionOverride = "A medium sized crystal outcrop of Obstructonium. Incredibly reflective.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/ObstructoniumNodeView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ObstructoniumNode.png");
                        Instance.FixColliders();

                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";

                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 7.5f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 7.5f;
                        Instance.GetComponent<PhysicalBehaviour>().rigidbody.mass = 7.5f;

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Glass");
                        phys.Properties.Flammability = 0f;
                    }
                }
            );
            //Ostrumian Metal Table
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Pole"),
                    NameOverride = "Ostrumian Metal Table -OF",
                    NameToOrderByOverride = "AA",
                    DescriptionOverride = "A table made of metal.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/MetalTableView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/MetalTable.png");

                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.75f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.75f;
                        Instance.GetComponent<PhysicalBehaviour>().rigidbody.mass = 0.75f;

                        foreach (var c in Instance.GetComponents<Collider2D>())
                        {
                            GameObject.Destroy(c);
                        }
                        var Collision1 = Instance.AddComponent<BoxCollider2D>();
                        Collision1.size = new Vector2(2.057143f, 0.08571416f);
                        Collision1.offset = new Vector2(0f, 0.4285715f);
                        var Collision2 = Instance.AddComponent<BoxCollider2D>();
                        Collision2.size = new Vector2(0.05714262f, 0.857143f);
                        Collision2.offset = new Vector2(-0.9142851f, -0.04285708f);
                        var Collision3 = Instance.AddComponent<BoxCollider2D>();
                        Collision3.size = new Vector2(0.05714262f, 0.857143f);
                        Collision3.offset = new Vector2(0.9142851f, -0.04285708f);
                    }
                }
            );
            //Ostrumian Metal Chair
            ModAPI.Register(
                new Modification()
                    {
                    OriginalItem = ModAPI.FindSpawnable("Metal Pole"),
                    NameOverride = "Ostrumian Metal Chair -OF",
                    NameToOrderByOverride = "AAA",
                    DescriptionOverride = "A chair made of metal.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/MetalChairView.png"),
                    AfterSpawn = (Instance) =>
                    {
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/MetalChair.png");

                    Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.4f;
                    Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.4f;
                    Instance.GetComponent<PhysicalBehaviour>().rigidbody.mass = 0.4f;

                        foreach (var c in Instance.GetComponents<Collider2D>())
                        {
                            GameObject.Destroy(c);
                        }
                        var Collision1 = Instance.AddComponent<BoxCollider2D>();
                        Collision1.size = new Vector2(0.0857144f, 0.885714f);
                        Collision1.offset = new Vector2(-0.2714285f, 0.2714287f);
                        var Collision2 = Instance.AddComponent<BoxCollider2D>();
                        Collision2.size = new Vector2(0.5428571f, 0.08571404f);
                        Collision2.offset = new Vector2(0.04285723f, -0.1285712f);
                        var Collision3 = Instance.AddComponent<BoxCollider2D>();
                        Collision3.size = new Vector2(0.05714273f, 0.5428575f);
                        Collision3.offset = new Vector2(-0.2285712f, -0.442857f);
                        var Collision4 = Instance.AddComponent<BoxCollider2D>();
                        Collision4.size = new Vector2(0.05714273f, 0.5428575f);
                        Collision4.offset = new Vector2(0.2285712f, -0.442857f);
                    }
                }
            );
            //Ostrumian Handheld Mining Drill
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Chainsaw"),
                    NameOverride = "Ostrumian Handheld Mining Drill -OF",
                    NameToOrderByOverride = "T",
                    DescriptionOverride = "drill go brrrrr",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/HandheldDrillView.png"),
                    AfterSpawn = (Instance) =>
                    {

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/HandHeldDrill.png");

                    }
                }
            );
            //Ostrumian Mining Drill
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Chainsaw"),
                    NameOverride = "Ostrumian Mining Drill -OF",
                    NameToOrderByOverride = "UU",
                    DescriptionOverride = "DRILL GO BRRRRR",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/DrillView.png"),
                    AfterSpawn = (Instance) =>
                    {

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Drill.png");

                    }
                }
            );
            //Ostrumian Combat Radio
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Ostrumian Combat Radio -OF",
                    NameToOrderByOverride = "TT",
                    DescriptionOverride = "A small handheld device that allows people to communicate through radio waves.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/CombatRadioView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/CombatRadio.png");
                        Instance.FixColliders();

                        AudioSource sound = Instance.AddComponent<AudioSource>();
                        sound.minDistance = 1;
                        sound.maxDistance = 1;
                        int index = 0;
                        AudioClip[] AudioData = new AudioClip[]
                        {
                            ModAPI.LoadSound("Radio1.wav"),
                            ModAPI.LoadSound("Radio2.wav"),
                            ModAPI.LoadSound("Radio3.wav"),
                            ModAPI.LoadSound("Radio4.wav"),
                        };
                        Instance.AddComponent<UseEventTrigger>().Action = () =>
                        {
                            sound.clip = AudioData[index];
                            sound.Play();
                            index++;
                            if (index == AudioData.Length) index = 0;
                        };
                        Instance.GetComponent<GlowingHotMetalBehaviour>().enabled = false;
                    }
                }
            );
            //Ostrumian Estrogen Pills
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Ostrumian Estrogen Pills -OF",
                    NameToOrderByOverride = "ZZZZZZZZZZZZZZZZZZ",
                    DescriptionOverride = "Congrats, you found the secret! This item is here because of an inside joke in the Arcadia Discord server about how all Ostrumians are, apparently, femboys.",
                    CategoryOverride = ModAPI.FindCategory("? -OF"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/FuseView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Z.png");
                        Instance.FixColliders();
                        Instance.GetComponent<GlowingHotMetalBehaviour>().enabled = false;

                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.1f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.1f;
                        Instance.GetComponent<PhysicalBehaviour>().rigidbody.mass = 0.1f;
                    }
                }
            );
            //Ostrumian Mini Thruster
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Mini Thruster"),
                    NameOverride = "Ostrumian Mini Thruster -OF",
                    NameToOrderByOverride = "KK",
                    DescriptionOverride = "A thruster, but smaller.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/MiniThrusterView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/MiniThruster.png");
                    }
                }
            );
            //Ostrumian Fire Extinguisher
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Fire Extinguisher"),
                    NameOverride = "Ostrumian Fire Extinguisher -OF",
                    NameToOrderByOverride = "S",
                    DescriptionOverride = "Safety first!",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/FireExtinguisherView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/FireExtinguisher.png");
                    }
                }
            );
            //Ostrumian Ion Thruster
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Ion Thruster"),
                    NameOverride = "Ostrumian Ion Thruster -OF",
                    NameToOrderByOverride = "KKKK",
                    DescriptionOverride = "'We don't care how efficient or powerful it is, look at this thing! It's awesome!' - Unnamed Ostrumian Engineer",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/IonThrusterView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/IonThruster.png");
                   	Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Piston
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Piston"),
                    NameOverride = "Ostrumian Piston -OF",
                    NameToOrderByOverride = "JJJ",
                    DescriptionOverride = "Pushes things.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/PistonView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Piston.png");
                        Instance.transform.GetChild(0).GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/PistonHead.png");
                    }
                }
            );
            //Ostrumian Generator
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Generator"),
                    NameOverride = "Ostrumian Generator -OF",
                    NameToOrderByOverride = "GGG",
                    DescriptionOverride = "Generates electricity. That's why it's called a generator.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/GeneratorView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Generator.png");
                    }
                }
            );
            //Ostrumian Industrial Generator
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Industrial Generator"),
                    NameOverride = "Industrial Ostrumian Generator -OF",
                    NameToOrderByOverride = "GGGG",
                    DescriptionOverride = "A much more powerful version of the generator. It does shake quite a bit though, so not ideal for anything that moves.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/BigGeneratorView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/BigGenerator.png");
                    }
                }
            );
            //Ostrumian Rotor
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rotor"),
                    NameOverride = "Ostrumian Rotor -OF",
                    NameToOrderByOverride = "SS",
                    DescriptionOverride = "Spin :)",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/RotorView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ServoBase.png");
                        Instance.transform.GetChild(0).GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Rotor.png");
                    }
                }
            );
            //Ostrumian Conveyer Belt
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Conveyor Belt"),
                    NameOverride = "Ostrumian Conveyor Belt -OF",
                    NameToOrderByOverride = "SA",
                    DescriptionOverride = "Pushes things along a mechanical belt.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/ConveyerBeltView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ConveyerBelt.png");
                        Instance.transform.GetChild(0).GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ConveyerWheel.png");
                        Instance.transform.GetChild(1).GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ConveyerWheel.png");
                        Instance.transform.GetChild(2).GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ConveyerWheel.png");
                        Instance.transform.GetChild(3).GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ConveyerWheel.png");
                        Instance.transform.GetChild(4).GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ConveyerWheel.png");

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Insulator");
                    }
                }
            );
            //Ostrumian Servo
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Servo"),
                    NameOverride = "Ostrumian Servo -OF",
                    NameToOrderByOverride = "SSS",
                    DescriptionOverride = "Spin :) but it stops after a little :(",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/ServoView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ServoBase.png");
                        Instance.transform.GetChild(0).GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Servo.png");
                        Instance.transform.GetChild(1).GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ServoArm.png");
                    }
                }
            );
            //Ostrumian Electron Sword
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Energy Sword"),
                    NameOverride = "Ostrumian Electron Sword -OF",
                    NameToOrderByOverride = "R",
                    DescriptionOverride = "Creates a concentrated field of electrons out of the hilt. Expensive. Uses Transvorium and Prolotosium.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/ElectronSwordView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ElectronSword.png");
                        Instance.transform.GetChild(0).GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ElectronSwordBlade.png");
                    }
                }
            );
            //Ostrumian Thruster
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Thruster"),
                    NameOverride = "Ostrumian Thruster -OF",
                    NameToOrderByOverride = "KKK", //i swear its not what it looks like
                    DescriptionOverride = "Thrusts.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/ThrusterView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Thruster.png");
                    }
                }
            );
            //Ostrumian Battery
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Battery"),
                    NameOverride = "Ostrumian Battery -OF",
                    NameToOrderByOverride = "G",
                    DescriptionOverride = "A small energy generating device.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/BatteryView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Battery.png");
                    }
                }
            );
            //Ostrumian Accumulator
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Accumulator"),
                    NameOverride = "Ostrumian Accumulator -OF",
                    NameToOrderByOverride = "GG",
                    DescriptionOverride = "A larger energy generating device.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/HeavyBatteryView.png"),
                    AfterSpawn = (Instance) =>
                    {
                     Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/HeavyBattery.png");
             	     Instance.FixColliders();

                     var phys = Instance.GetComponent<PhysicalBehaviour>();
                     phys.Properties = ModAPI.FindPhysicalProperties("Metal");
                    }
                }
            );
            //Ostrumian Wheel
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Motorised Wheel"),
                    NameOverride = "Ostrumian Motorised Wheel -OF",
                    NameToOrderByOverride = "S",
                    DescriptionOverride = "Like a rotor but you can't set the speed and it has suspension.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/MotorView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Motor.png");
                        Instance.transform.GetChild(0).GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/MotorWheel.png");

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Metal");
                    }
                }
            );
            //Ostrumian Chair
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Bus Chair"),
                    NameOverride = "Ostrumian Chair -OF",
                    NameToOrderByOverride = "L",
                    DescriptionOverride = "A chair. Uses nanotechnology and carbon fibres combined with graphene to create the most durable and comfortable chair ever. Just kidding. It's a chair.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/SeatView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                        {
                            ModAPI.LoadSprite("sprite/Seat.png"),
                        };
                    }
                }
            );
            //Ostrumian Barrel
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Red Barrel"),
                    NameOverride = "Ostrumian Barrel -OF",
                    NameToOrderByOverride = "IIIIII",
                    DescriptionOverride = "A barrel that holds very flammable substance. Might be a good idea to keep it away from fire.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/RedBarrelView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/RedBarrel.png");
                        	Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Gastank
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Red Barrel"),
                    NameOverride = "Ostrumian Gastank -OF",
                    NameToOrderByOverride = "IIII",
                    DescriptionOverride = "A gastank for vehicles.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/GastankView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Gastank.png");
                        Instance.FixColliders();

                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 1.5f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 1.5f;
                        Instance.GetComponent<PhysicalBehaviour>().rigidbody.mass = 1.5f;
                    }
                }
            );
            //Ostrumian Large Gastank
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Red Barrel"),
                    NameOverride = "Ostrumian Large Gastank -OF",
                    NameToOrderByOverride = "IIIII",
                    DescriptionOverride = "A large gastank for large vehicles.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/LargeGastankView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/LargeGastank.png");
                        Instance.FixColliders();

                        Instance.GetComponent<ExplosiveBehaviour>().Range = 12.5f;

                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 10f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 10f;
                        Instance.GetComponent<PhysicalBehaviour>().rigidbody.mass = 10f;
                    }
                }
            );
            //Ostrumian Engine
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Small I-Beam"),
                    NameOverride = "Ostrumian Engine -OF",
                    NameToOrderByOverride = "II",
                    DescriptionOverride = "VROOM VROOM BITCHES",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/EngineView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Engine.png");
                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Large Engine
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Small I-Beam"),
                    NameOverride = "Ostrumian Large Engine -OF",
                    NameToOrderByOverride = "III",
                    DescriptionOverride = "Can you rev the engine of a tank?",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/LargeEngineView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/LargeEngine.png");
                        Instance.FixColliders();
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 10f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 10f;
                        Instance.GetComponent<PhysicalBehaviour>().rigidbody.mass = 10f;
                    }
                }
            );
            //Ostrumian Electric Engine
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Small I-Beam"),
                    NameOverride = "Ostrumian Electric Engine -OF",
                    NameToOrderByOverride = "I",
                    DescriptionOverride = "Eco-friendly! (Kind of)",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/ElectricEngineView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ElectricEngine.png");
                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Smoke Grenade
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Activation Fuse"),
                    NameOverride = "Ostrumian Smoke Grenade -OF",
                    NameToOrderByOverride = "M",
                    DescriptionOverride = "A device that explodes releasing a cloud of white smoke.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/SmokeGrenadeView.png"),
                    AfterSpawn = (Instance) =>
                    {

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/SmokeGrenade.png");
                        Instance.FixColliders();
                        Instance.AddComponent<UseEventTrigger>().Action = () =>
                        {
                            var e = ModAPI.CreateParticleEffect("BigExplosion", Instance.transform.position);
                            var m = e.GetComponent<ParticleSystem>().main;
                            m.startColor = new Color(202f, 202f, 202f);
                            var Colder = e.GetComponent<ParticleSystem>().main;
                            Colder.startColor = new Color(202f, 202f, 202f);
                        };
                    }
                }
            );
            //Ostrumian Flashbang
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Activation Fuse"),
                    NameOverride = "Ostrumian Flashbang -OF",
                    NameToOrderByOverride = "L",
                    DescriptionOverride = "A device that explodes releasing a blinding flash of white light.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/FlashbangView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Flashbang.png");
                        Instance.AddComponent<UseEventTrigger>().Action = () => {
                            Instance.AddComponent<FlashbangFlash>();
                        };
                        Instance.FixColliders();

                    }
                }
            );
            //Ostrumian Knife
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Knife"),
                    NameOverride = "Ostrumian Knife -OF",
                    NameToOrderByOverride = "B",
                    DescriptionOverride = "It's a knife. Not much to say about it.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/KnifeView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                     Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Knife.png", 1f);
                   	Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Rifle
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Assault Rifle"),
                    NameOverride = "Ostrumian Rifle -OF",
                    NameToOrderByOverride = "D",
                    DescriptionOverride = "A standard issue assault rifle given to soldiers.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/RifleView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Rifle.png", 1f);
                   	    Instance.FixColliders();

                        var firearm = Instance.GetComponent<FirearmBehaviour>();
                        firearm.Automatic = true;
                        firearm.InitialInaccuracy = 0.001f;

                        Cartridge customCartridge = ModAPI.FindCartridge("5.56x45mm");
                        customCartridge.Damage *= 1.8f;
                        customCartridge.PenetrationRandomAngleMultiplier *= 0f;
                        customCartridge.StartSpeed *= 1.2f;
                        customCartridge.ImpactForce = 0.2f;
                        customCartridge.Recoil *= 0.8f;
                        firearm.Cartridge = customCartridge;
                    }
                }
            );
            //Ostrumian Heavy Rifle
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("AK-47"),
                    NameOverride = "P5-AR -OF",
                    NameToOrderByOverride = "TTa",
                    DescriptionOverride = "A modified Ostrumian assault rifle. Slower firerate, but packs a bigger punch.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/HeavyRifleView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/HeavyRifle.png", 1f);
                        Instance.FixColliders();

                        var firearm = Instance.GetComponent<FirearmBehaviour>();
                        
                        firearm.barrelPosition = new Vector3(0.70f, 0.1f);
                        firearm.Automatic = true;
                        firearm.AutomaticFireInterval = 0.15f;
                        firearm.InitialInaccuracy = 0.005f;

                        Cartridge customCartridge = ModAPI.FindCartridge("7.62x39mm");
                        customCartridge.name = "Heavy rifle cartridge";
                        customCartridge.Damage *= 2.3f;
                        customCartridge.PenetrationRandomAngleMultiplier *= 0f;
                        customCartridge.StartSpeed *= 1f;
                        customCartridge.ImpactForce = 0.4f;
                        customCartridge.Recoil *= 1.3f;
                        firearm.Cartridge = customCartridge;
                    }
                }
            );
            //Ostrumian SMG
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Submachine Gun"),
                    NameOverride = "Ostrumian Submachine Gun -OF",
                    NameToOrderByOverride = "DD",
                    DescriptionOverride = "A standard lightweight SMG. Compact and portable.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/SMGView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/SMG.png", 1f);
                        Instance.FixColliders();

                        var firearm = Instance.GetComponent<FirearmBehaviour>();

                        Cartridge customCartridge = ModAPI.FindCartridge("9mm");
                        customCartridge.name = "SMG cartridge";
                        firearm.AutomaticFireInterval = 0.065f;
                        firearm.Automatic = true;
                    }
                }
            );
            //C-64
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Submachine Gun"),
                    NameOverride = "C-64 -OF",
                    NameToOrderByOverride = "TTb",
                    DescriptionOverride = "A lightweight compact assault rifle. Commonly used by Gravimancers.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/CarbineView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Carbine.png", 1f);
                        Instance.FixColliders();

                        var firearm = Instance.GetComponent<FirearmBehaviour>();
                        firearm.Automatic = true;
                        firearm.InitialInaccuracy = 0.001f;

                        Cartridge customCartridge = ModAPI.FindCartridge("5.56x45mm");
                        customCartridge.Damage *= 1.1f;
                        customCartridge.PenetrationRandomAngleMultiplier *= 0f;
                        customCartridge.StartSpeed *= 1.1f;
                        customCartridge.ImpactForce = 0.3f;
                        customCartridge.Recoil *= 0.3f;
                        firearm.Cartridge = customCartridge;
                    }
                }
            );
            //Ostrumian Anti-grav Grenade
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Dynamite"),
                    NameOverride = "Ostrumian Gravity Spike Grenade -OF",
                    NameToOrderByOverride = "TTbb",
                    DescriptionOverride = "A handheld thowable explosive that uses Gravitasium to launch objects and people into the air.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/AntiGravGrenadeView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/AntiGravGrenade.png");
                        Instance.FixColliders();

                        Instance.GetComponent<ExplosiveBehaviour>().Range = 5f;
                        Instance.GetComponent<ExplosiveBehaviour>().Delay = 5f;
                        Instance.GetComponent<ExplosiveBehaviour>().FragmentationRayCount = 32;
                        Instance.GetComponent<ExplosiveBehaviour>().BallisticShrapnelCount = 0;
                        Instance.GetComponent<ExplosiveBehaviour>().FragmentForce = 2.25f;
                        Instance.GetComponent<ExplosiveBehaviour>().ShockwaveLiftForce = 375f;
                        Instance.GetComponent<ExplosiveBehaviour>().ShockwaveStrength = 625f;
                        Instance.GetComponent<ExplosiveBehaviour>().BurnPower = 0f;
                        Instance.GetComponent<ExplosiveBehaviour>().ShotExplodeChance = 0f;
                        Instance.GetComponent<ExplosiveBehaviour>().DismemberChance = 0f;
                        Instance.GetComponent<ExplosiveBehaviour>().ExplodeBurnProgressThreshold = 0f;
                        Instance.GetComponent<ExplosiveBehaviour>().BigExplosion = false;
                        Instance.GetComponent<ExplosiveBehaviour>().ParticleEffectAndSound = false;

                        Instance.AddComponent<ParticleOnExplode>();

                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.1f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.1f;
                        Instance.GetComponent<PhysicalBehaviour>().rigidbody.mass = 0.1f;
                        AudioSource sound = Instance.AddComponent<AudioSource>();
                        sound.minDistance = 1;
                        sound.maxDistance = 1;
                        int index = 0;
                        AudioClip[] AudioData = new AudioClip[]
                        {
                            ModAPI.LoadSound("gravgrenadebeep.wav"),
                        };
                        Instance.AddComponent<UseEventTrigger>().Action = () =>
                        {
                            sound.clip = AudioData[index];
                            sound.Play();
                            index++;
                            if (index == AudioData.Length) index = 0;
                        };

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Bulletproof");
                        phys.rigidbody.gravityScale *= 0.5f;

                        var item = Instance.GetComponent<PhysicalBehaviour>();
                        item.HoldingPositions = new[]
                        {
                            new Vector3(0f, 0f)
                        };
                    }
                }
            );
            //Ostrumian Grav Grenade
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Dynamite"),
                    NameOverride = "Ostrumian Gravity Well Grenade -OF",
                    NameToOrderByOverride = "TTbb",
                    DescriptionOverride = "A handheld thowable explosive that uses Gravitasium to pull people towards the ground.",
                    CategoryOverride = ModAPI.FindCategory("? -OF"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/AntiGravGrenadeView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/GravGrenade.png");
                        Instance.FixColliders();

                        Instance.GetComponent<ExplosiveBehaviour>().Range = 10f;
                        Instance.GetComponent<ExplosiveBehaviour>().Delay = 5f;
                        Instance.GetComponent<ExplosiveBehaviour>().FragmentationRayCount = 32;
                        Instance.GetComponent<ExplosiveBehaviour>().BallisticShrapnelCount = 0;
                        Instance.GetComponent<ExplosiveBehaviour>().FragmentForce = -0.5f;
                        Instance.GetComponent<ExplosiveBehaviour>().ShockwaveLiftForce = -3000f;
                        Instance.GetComponent<ExplosiveBehaviour>().ShockwaveStrength = 625f;
                        Instance.GetComponent<ExplosiveBehaviour>().BurnPower = 0f;
                        Instance.GetComponent<ExplosiveBehaviour>().ShotExplodeChance = 0f;
                        Instance.GetComponent<ExplosiveBehaviour>().DismemberChance = 0.5f;
                        Instance.GetComponent<ExplosiveBehaviour>().ExplodeBurnProgressThreshold = 0f;
                        Instance.GetComponent<ExplosiveBehaviour>().BigExplosion = false;
                        Instance.GetComponent<ExplosiveBehaviour>().ParticleEffectAndSound = false;

                        Instance.AddComponent<ParticleOnExplode>();

                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.1f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.1f;
                        Instance.GetComponent<PhysicalBehaviour>().rigidbody.mass = 0.1f;
                        AudioSource sound = Instance.AddComponent<AudioSource>();
                        sound.minDistance = 1;
                        sound.maxDistance = 1;
                        int index = 0;
                        AudioClip[] AudioData = new AudioClip[]
                        {
                            ModAPI.LoadSound("gravgrenadebeep.wav"),
                        };
                        Instance.AddComponent<UseEventTrigger>().Action = () =>
                        {
                            sound.clip = AudioData[index];
                            sound.Play();
                            index++;
                            if (index == AudioData.Length) index = 0;
                        };

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Bulletproof");
                        phys.rigidbody.gravityScale *= 0.5f;

                        var item = Instance.GetComponent<PhysicalBehaviour>();
                        item.HoldingPositions = new[]
                        {
                            new Vector3(0f, 0f)
                        };
                    }
                }
            );
            //Ostrumian Breach Charge
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Dynamite"),
                    NameOverride = "Ostrumian Breach Charge -OF",
                    NameToOrderByOverride = "Ma",
                    DescriptionOverride = "An explosive device with a very small range, but packs quite the punch",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/BreachChargeView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/BreachCharge.png");
                        Instance.FixColliders();

                        var item = Instance.GetComponent<PhysicalBehaviour>();
                        item.Properties = ModAPI.FindPhysicalProperties("Bulletproof");

                        Instance.GetComponent<ExplosiveBehaviour>().Range = 1f;
                        Instance.GetComponent<ExplosiveBehaviour>().Delay = 0f;
                        Instance.GetComponent<ExplosiveBehaviour>().ExplodeBurnProgressThreshold = 99f;
                        Instance.GetComponent<ExplosiveBehaviour>().ExplodesOnFragmentHit = false;
                        Instance.GetComponent<ExplosiveBehaviour>().ShotExplodeChance = 0f;
                        Instance.GetComponent<ExplosiveBehaviour>().BurnPower = 100f;
                        Instance.GetComponent<ExplosiveBehaviour>().DismemberChance = 1f;
                    }
                }
            );
            //Ostrumian CGPE
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Pulse Drum"),
                    NameOverride = "Ostrumian CGPE -OF",
                    NameToOrderByOverride = "TTba",
                    DescriptionOverride = "An underbarrel attachment for a weapon. Short for Compact Gravitational Pulse Emitter, and is usually used by Gravimancers.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/PulseView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Pulse.png", 1f);
                        Instance.FixColliders();

                        var pulse = Instance.GetComponent<PulseDrumBehaviour>();

                        pulse.barrelPosition = new Vector2(0f, 0f);

                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.1f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.1f;
                        Instance.GetComponent<PhysicalBehaviour>().rigidbody.mass = 0.1f;

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Insulator");
                    }
                }
            );
            //Ostrumian Light Machine Gun
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Light Machine Gun"),
                    NameOverride = "Ostrumian Light Machine Gun -OF",
                    NameToOrderByOverride = "EE",
                    DescriptionOverride = "A standard LMG given to heavy soldiers. Sometimes mounted on vehicles.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/LightMachineGunView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/LightMachineGun.png", 1f);
                        Instance.FixColliders();

                        var firearm = Instance.GetComponent<FirearmBehaviour>();
                        
                        firearm.Automatic = true;
                        firearm.AutomaticFireInterval = 0.1f;
                        firearm.InitialInaccuracy = 0.008f;

                        Cartridge customCartridge = ModAPI.FindCartridge("7.62x51mm");
                        customCartridge.name = "LMG cartridge";
                        customCartridge.Damage *= 3.3f;
                        customCartridge.PenetrationRandomAngleMultiplier *= 0f;
                        customCartridge.StartSpeed *= 1.3f;
                        customCartridge.ImpactForce = 0.6f;
                        customCartridge.Recoil *= 1.6f;
                        firearm.Cartridge = customCartridge;
                    }
                }
            );
            //Ostrumian Flamethrower
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Flamethrower"),
                    NameOverride = "Ostrumian Flamethrower -OF",
                    NameToOrderByOverride = "F",
                    DescriptionOverride = "Throws flames. A surprisingly effective weapon, although the morality of using it is often questioned.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/FlamethrowerView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Flamethrower.png", 1f);
                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Handheld Rocket Launcher
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rocket Launcher"),
                    NameOverride = "Ostrumian Handheld Rocket Launcher -OF",
                    NameToOrderByOverride = "GG",
                    DescriptionOverride = "A handheld version of the rocket silo. Fires unguided rocket-propelled grenades.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/HandHeldRocketLauncherView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/HandHeldRocketLauncher.png", 1f);
                        Instance.FixColliders();
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Bottom";
                    }
                }
            );
            //Ostrumian Rocket Silo
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rocket Launcher"),
                    NameOverride = "Ostrumian Rocket Silo -OF",
                    NameToOrderByOverride = "9",
                    DescriptionOverride = "Fires unguided rocket-propelled grenades. Used on mechs, starships and some vehicles.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/RocketSiloView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/RocketSilo.png", 1f);
                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Missile Silo
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Missile Launcher"),
                    NameOverride = "Ostrumian Missile Silo -OF",
                    NameToOrderByOverride = "9",
                    DescriptionOverride = "Fires guided rocket-propelled grenades. Used starships.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/MissileSiloView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/MissileSilo.png", 1f);
                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian 120mm Cannon
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Detached 120mm Cannon"),
                    NameOverride = "Ostrumian 120mm Cannon -OF",
                    NameToOrderByOverride = "9",
                    DescriptionOverride = "Shoots massive 120mm rounds. Used in tanks and starships.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/120mmCannonView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                     Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/120mmCannon.png", 1f);
                   	Instance.FixColliders();
                    }
                }
            );
            //Ostrumian 30mm HEAT Cannon
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Detached 30mm HEAT Cannon"),
                    NameOverride = "Ostrumian 30mm HEAT Cannon -OF",
                    NameToOrderByOverride = "9",
                    DescriptionOverride = "Shoots 30mm explosive rounds. Used in tanks and starships.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/HEATCannonView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/HEATCannon.png", 1f);
                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Plasma Bolt Cannon
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Detached Laser Cannon"),
                    NameOverride = "Ostrumian Plasma Bolt Cannon -OF",
                    NameToOrderByOverride = "9",
                    DescriptionOverride = "Shoots very high pressure bolts of ionized plasma. Used to take out external constructs like weapons, antenna, and other exposed weak spots. pUsed on starships and, in some rare cases, tanks. Uses Transvorium and Prolotosium.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/LaserCannonView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/LaserCannon.png", 1f);
                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian 20mm
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Sniper Rifle"),
                    NameOverride = "Ostrumian Mounted 20mm -OF",
                    NameToOrderByOverride = "5",
                    DescriptionOverride = "A heavy automatic mounted machine gun. Mounted on small military vehicles.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/20mmView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        var firearm = Instance.GetComponent<FirearmBehaviour>();
                        firearm.barrelPosition = new Vector3(0.3f, 0f);

                        Cartridge customCartridge = ModAPI.FindCartridge("30 mm");
                        customCartridge.name = "Ostrumian 30mm";
                        firearm.AutomaticFireInterval = 0.05f;
                        firearm.Automatic = true;

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/20mm.png", 1f);
                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Shotgun
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Shotgun"),
                    NameOverride = "Ostrumian Shotgun -OF",
                    NameToOrderByOverride = "D",
                    DescriptionOverride = "A tactical shotgun that's branded with the Ostrumian Federation. It's basically no different from a regular one.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/ShotgunView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Shotgun.png", 1f);
                   	    Instance.FixColliders();

                        var firearm = Instance.GetComponent<FirearmBehaviour>();

                        firearm.InitialInaccuracy = 0.1f;
                        firearm.BulletsPerShot = 8;

                        Cartridge customCartridge = ModAPI.FindCartridge("7.62x51mm");
                        customCartridge.name = "Shotgun cartridge";
                        customCartridge.Damage *= 6f;
                        customCartridge.PenetrationRandomAngleMultiplier *= 0f;
                        customCartridge.StartSpeed *= 1f;
                        customCartridge.ImpactForce = 0.9f;
                        customCartridge.Recoil *= 0.6f;
                        firearm.Cartridge = customCartridge;
                    }
                }
            );
            //Ostrumian Sniper Rifle
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Sniper Rifle"),
                    NameOverride = "Ostrumian Sniper Rifle -OF",
                    NameToOrderByOverride = "E",
                    DescriptionOverride = "A strong long ranged rifle with a scope.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/SniperRifleView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Sniper.png", 1f);
                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Grenade
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Handgrenade"),
                    NameOverride = "Ostrumian Grenade -OF",
                    NameToOrderByOverride = "J",
                    DescriptionOverride = "A small handheld throwable explosive. Who the hell thought 'hey so you know those bombs? what if we shrunk them and gave them to soldiers to throw'?",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/GrenadeView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Grenade.png", 1f);
                   	    Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Sticky Grenade
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Sticky Grenade"),
                    NameOverride = "Ostrumian Sticky Grenade -OF",
                    NameToOrderByOverride = "K",
                    DescriptionOverride = "A regular grenade that, once activated, removes a protective outer shell that reveals a rubbery surface on the inside that sticks to almost anything.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/StickyGrenadeView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/StickyGrenade.png", 1f);
                   	    Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Pistol
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Glock 17"),
                    NameOverride = "Ostrumian Pistol -OF",
                    NameToOrderByOverride = "B",
                    DescriptionOverride = "A handgun. Nothing that special about it.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/PistolView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                     Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Pistol.png", 1f);
                   	Instance.FixColliders();
                    }
                }
            );
            //Peacekeeper Pistol
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Glock 17"),
                    NameOverride = "Ostrumian Peacekeeper Pistol -OF",
                    NameToOrderByOverride = "N",
                    DescriptionOverride = "A handgun. Nothing that special about it.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/PeacekeeperPistolView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/PeacekeeperPistol.png", 1f);
                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Grenade Launcher
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Grenade Launcher"),
                    NameOverride = "Ostrumian Grenade Launcher -OF",
                    NameToOrderByOverride = "G",
                    DescriptionOverride = "And y'know what? While we're at it, just shrink the regular ones and shove em in something that launches em!",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/GrenadeLauncherView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                     Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/GrenadeLauncher.png", 1f);
                   	Instance.FixColliders();
                    }
                }
            );
            //P5-SC
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Submachine Gun"),
                    NameOverride = "P5-SC -OF",
                    NameToOrderByOverride = "DDa",
                    DescriptionOverride = "A modified submachine gun. Compact and portable. I made this gun but ended up not finding a proper use for it, so now it's hidden here.",
                    CategoryOverride = ModAPI.FindCategory("? -OF"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/SMGView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/CyberSMG.png", 1f);
                        Instance.FixColliders();

                        var firearm = Instance.GetComponent<FirearmBehaviour>();

                        Cartridge customCartridge = ModAPI.FindCartridge("9mm");
                        customCartridge.name = "P5-SC cartridge";
                        firearm.AutomaticFireInterval = 0.055f;
                        firearm.Automatic = true;
                    }
                }
            );
            //Ostrumian Wrench
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Wrench"),
                    NameOverride = "Ostrumian Wrench -OF",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "A wrench. Used to repair and build stuff, but also a formidable weapon.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/WrenchView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Wrench.png");
                        	Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Goggles
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Soap"),
                    NameOverride = "Ostrumian Goggles -OF",
                    NameToOrderByOverride = "VV",
                    DescriptionOverride = "Safety second!",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/GogglesView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Glass");
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Goggles.png");

                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 0f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 0;

                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Facemask
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Soap"),
                    NameOverride = "Ostrumian Facemask -OF",
                    NameToOrderByOverride = "V",
                    DescriptionOverride = "It's corona time.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/MaskView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Mask.png");

                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 0f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 0;

                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Civilian Helmet
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Ostrumian Civilian EVA Helmet -OF",
                    NameToOrderByOverride = "VVVV",
                    DescriptionOverride = "A standard helmet for civilians to pair with personal EVA Suits. Works with similar principles to their millitary counterparts.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/CivHelmetView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/CivilianHelmet.png");

                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 22f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 12;

                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Sailor Helmet
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Ostrumian Sailor Helmet -OF",
                    NameToOrderByOverride = "VVVVV",
                    DescriptionOverride = "A standard helmet for people in the navy.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/NavyHelmetView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/NavyHelmet.png");

                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 22f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 12;

                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Hardhat
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Ostrumian Hardhat -OF",
                    NameToOrderByOverride = "VVV",
                    DescriptionOverride = "A standard issue hardhat for labour workers.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/HardhatView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Hardhat.png");

                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 22f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 12;

                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Ballistic Upper
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Ostrumian Upper Armor -OF",
                    NameToOrderByOverride = "Y",
                    DescriptionOverride = "Helps to protect someone's chest.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/UpperArmourView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/UpperArmour.png");
                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 44f;
                        armor.armorPiece = "UpperBody";
                        armor.armorTier = 18;

                        Instance.FixColliders();
                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Bulletproof");
                    }
                }
            );
            //Ostrumian Ballistic Middle
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Ostrumiuan Middle Armor -OF",
                    NameToOrderByOverride = "YY",
                    DescriptionOverride = "Helps to protect someone's upper abdomen.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/LowerArmourView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/MiddleArmour.png");
                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 44f;
                        armor.armorPiece = "MiddleBody";
                        armor.armorTier = 18;

                        Instance.FixColliders();
                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Bulletproof");
                    }
                }
            );
            //Ostrumian Miner Helmet
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Ostrumian Miner Helmet -OF",
                    NameToOrderByOverride = "VVVVV",
                    DescriptionOverride = "A standard issue helmet for miners. Has more head protection.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/MinerHelmetView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/MinerHelmet.png");

                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 22f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 12;

                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Scientist EVA Helmet
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Ostrumian Scientist EVA Helmet -OF",
                    NameToOrderByOverride = "VVVVVV",
                    DescriptionOverride = "A standard issue helmet for scientists.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/ScientistEVAHelmetView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ScientistEVAHelmet.png");

                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 22f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 12;

                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Soldier Helmet
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Ostrumian Soldier Helmet -OF",
                    NameToOrderByOverride = "W",
                    DescriptionOverride = "A standard issue helmet given to regular soldiers.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/SoldierHelmet1View.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/SoldierHelmet1.png");

                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 22f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 12;

                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Peacekeeper Helmet
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Ostrumian Peacekeeper Helmet -OF",
                    NameToOrderByOverride = "W",
                    DescriptionOverride = "A standard issue helmet given to peacekeepers.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/PoliceHelmetView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/PoliceHelmet.png");

                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 22f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 12;

                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Captain Helmet
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Ostrumian Captain Helmet -OF",
                    NameToOrderByOverride = "XX",
                    DescriptionOverride = "A helmet given to captains.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/CommanderHelmetView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/CommanderHelmet.png");

                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 22f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 12;

                        Instance.FixColliders();
                    }
                }
            );
          //Ostrumian Pilot Helmet
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Ostrumian Pilot Helmet -OF",
                    NameToOrderByOverride = "WW",
                    DescriptionOverride = "A regular helmet fitted with a radio transceiver.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/PilotHelmetView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/PilotHelmet.png");

                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 22f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 12;

                        Instance.FixColliders();
                    }
                }
            );
          //Ostrumian Medic Helmet
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Ostrumian Medic Helmet -OF",
                    NameToOrderByOverride = "WWW",
                    DescriptionOverride = "A normal helmet with an AI built into it which can give a diagnosis on lifeforms.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/MedicHelmetView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/MedicHelmet.png");

                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 22f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 12;

                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Heavy Soldier Helmet
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Ostrumian Heavy Soldier Helmet -OF",
                    NameToOrderByOverride = "XXX",
                    DescriptionOverride = "A heavily protected helmet for heavy soldiers.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/HeavySoldierHelmetView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/HeavySoldierHelmet.png");
                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 22f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 12;

                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Sniper Helmet
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Ostrumian Sniper Helmet -OF",
                    NameToOrderByOverride = "WWWW",
                    DescriptionOverride = "A helmet equpped with Augmented Reality visors to aid snipers.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/SniperHelmetView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/SniperHelmet.png");

                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 22f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 12;

                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Energy Soldier Helmet
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Ostrumian Energy Soldier Helmet -OF",
                    NameToOrderByOverride = "XXXx",
                    DescriptionOverride = "A standard helmet used by Energy Soldiers",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/EnergySoldierHelmetView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/EnergySoldierHelmet.png");
                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 22f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 12;

                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Thermal Soldier Helmet
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Ostrumian Thermal Soldier Helmet -OF",
                    NameToOrderByOverride = "WWWWW",
                    DescriptionOverride = "A helmet that protects the user from high temperatures.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/ThermalSoldierHelmetView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ThermalSoldierHelmet.png");

                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 22f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 12;

                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Demolitions Expert Helmet
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Ostrumian Demolitions Expert Helmet -OF",
                    NameToOrderByOverride = "X",
                    DescriptionOverride = "A helmet that protects the user from explosions. Provides a wide range of view.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/DemoHelmetView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/DemoHelmet.png");

                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 22f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 12;

                        Instance.FixColliders();
                    }
                }
            );
            //Void Division Soldier Helmet
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Void Division Soldier Helmet -OF",
                    NameToOrderByOverride = "XXXX",
                    DescriptionOverride = "A very heavily protected helmet for soldiers in the Void Division.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/VoidDivSoldierHelmetView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/VoidDivSoldierHelmet.png");
                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 22f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 12;

                        Instance.FixColliders();

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Bulletproof");
                    }
                }
            );
            //Void Division Sniper Helmet
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Void Division Sniper Helmet -OF",
                    NameToOrderByOverride = "XXXXX",
                    DescriptionOverride = "A very heavily protected helmet for snipers in the Void Division. These helmets are equipped with advanced Augmented Reality visors to aid it's user.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/VoidDivSniperHelmetView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/VoidDivSniperHelmet.png");
                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 22f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 12;

                        Instance.FixColliders();
                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Bulletproof");
                    }
                }
            );
            //Cybersoldier Helmet
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Cybersoldier Helmet -OF",
                    NameToOrderByOverride = "XXXa",
                    DescriptionOverride = "A very heavily protected helmet for cybesoldiers. Activate to turn on the helmet light.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/CyberHelmetView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/CyberHelmet.png");
                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 22f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 12;

                        Instance.FixColliders();

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Bulletproof");
                    }
                }
            );
            //Gravimancer Helmet
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Gravimancer Helmet -OF",
                    NameToOrderByOverride = "XXXb",
                    DescriptionOverride = "A very heavily protected helmet for Gravimancers.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/GravimancerHelmetView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/GravimancerHelmet.png");
                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 22f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 12;

                        Instance.FixColliders();

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Bulletproof");
                        phys.rigidbody.gravityScale *= 0.5f;
                    }
                }
            );
            //Ostrumian Industrial Piston
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Industrial Piston"),
                    NameOverride = "Ostrumian Industrial Piston -OF",
                    NameToOrderByOverride = "JJJJ",
                    DescriptionOverride = "Pushes things, but its bigger and better.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/BigPistonView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/BigPiston.png");
                        Instance.transform.GetChild(0).GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/BigPistonHead.png");
                    }
                }
            );
            //Ostrumian Fuse
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Activation Fuse"),
                    NameOverride = "Ostrumian Activation Fuse -OF",
                    NameToOrderByOverride = "Q",
                    DescriptionOverride = "A single use activation fuse.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/FuseView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Fuse.png");
                    }
                }
            );
            //Ostrumian Activation Toggle
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Activation Toggle"),
                    NameOverride = "Ostrumian Activation Toggle -OF",
                    NameToOrderByOverride = "Qa",
                    DescriptionOverride = "A device that produces a constant activation output when toggled.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/ActivationToggleView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ActivationToggle.png");
                    }
                }
            );
            //Ostrumian Transvorium Blaster
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Stunner"),
                    NameOverride = "Ostrumian Transvorium Blaster -OF",
                    NameToOrderByOverride = "O",
                    DescriptionOverride = "Shoots out a ball of electricity. Used for non-lethally disarming agressors. Used by Peacekeepers.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/StunView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Stun.png", 1f);
                   	    Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Mounted Transvorium Blaster
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Stunner"),
                    NameOverride = "Ostrumian Mounted Transvorium Blaster -OF",
                    NameToOrderByOverride = "Oa",
                    DescriptionOverride = "Shoots out a ball of electricity. Used for non-lethally disarming agressors. Not handheld.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/MountedStunView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/MountedStun.png", 1f);
                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Exotic Generator
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "Ostrumian Exotic Generator -OF",
                    NameToOrderByOverride = "HHH",
                    DescriptionOverride = "Generates similar energy to an industrial generator. Fueled by exotic matter. Doesn't shake.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/MiniExoticReactorView.png"),
                    AfterSpawn = (Instance) =>
                    {
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/MiniExoticReactor.png");
                   	Instance.GetComponent<GlowingHotMetalBehaviour>().enabled = false;

            		var item = Instance.GetComponent<PhysicalBehaviour>();
                    item.Properties = ModAPI.FindPhysicalProperties("Insulator");
                    item.ForceNoChargeParticles = true;
                   	Instance.FixColliders();
                        Instance.GetOrAddComponent<NoTemperature>();
                        if (Instance.GetComponent<AfterSpawnBehaviour>() == null)
                        {
                            Instance.AddComponent<AfterSpawnBehaviour>();
                            Instance.AddComponent<CustomGlowBattery>();
                        }

                        Instance.GetComponent<CustomGlowBattery>().TargetCharge = 1000f;
                        Instance.GetComponent<CustomGlowBattery>().GlowSprite = ModAPI.LoadSprite("sprite/MiniExoticReactorMask.png");
                        Instance.GetComponent<CustomGlowBattery>().Colour = new Color32(50, 50, 200, 100);
                    }
                }
            );
            //Ostrumian Exotic Battery
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Small I-Beam"),
                    NameOverride = "Ostrumian Exotic Battery -OF",
                    NameToOrderByOverride = "HH",
                    DescriptionOverride = "A much smaller version of the Exotic Generator. Generates an energy output between that of a Generator and Industrial Generator.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/ExoticBatteryView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ExoticBattery.png");
                        Instance.GetComponent<GlowingHotMetalBehaviour>().enabled = false;

                        var item = Instance.GetComponent<PhysicalBehaviour>();
                        item.Properties = ModAPI.FindPhysicalProperties("Insulator");
                        item.ForceNoChargeParticles = true;
                        Instance.FixColliders();
                        Instance.GetOrAddComponent<NoTemperature>();
                        if (Instance.GetComponent<AfterSpawnBehaviour>() == null)
                        {
                            Instance.AddComponent<AfterSpawnBehaviour>();
                            Instance.AddComponent<CustomGlowBattery>();
                        }

                        Instance.GetComponent<CustomGlowBattery>().TargetCharge = 400f;
                        Instance.GetComponent<CustomGlowBattery>().GlowSprite = ModAPI.LoadSprite("sprite/ExoticBatteryMask.png");
                        Instance.GetComponent<CustomGlowBattery>().Colour = new Color32(50, 50, 200, 100);
                    }
                } 
            );
            //Ostrumian Shield Generator
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Mirror"),
                    NameOverride = "Ostrumian Shield Generator -OF",
                    NameToOrderByOverride = "BBBBB",
                    DescriptionOverride = "A panel that creates a field of bulletproof electrons. When deactivated, it automatically has no collision.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/ShieldGeneratorView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ShieldGenerator.png");
                        Instance.GetComponent<SpriteRenderer>().material = ModAPI.FindMaterial("Sprites-Default");

                        foreach (var c in Instance.GetComponents<Collider2D>())
                        {
                            GameObject.Destroy(c);
                        }
                        Instance.AddComponent<BoxCollider2D>();

                        if (Instance.GetComponent<AfterSpawnBehaviour>() == null)
                        {
                            Instance.AddComponent<AfterSpawnBehaviour>();
                            Instance.AddComponent<CustomGlowBattery>();
                        }

                        Instance.GetComponent<CustomGlowBattery>().TargetCharge = 10f;
                        Instance.GetComponent<CustomGlowBattery>().GlowSprite = ModAPI.LoadSprite("sprite/ShieldGeneratorMask.png");
                        Instance.GetComponent<CustomGlowBattery>().Colour = new Color32(255, 220, 0, 50);

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.ForceNoChargeParticles = true;
                        phys.SimulateTemperature = false;
                        phys.Properties = ModAPI.FindPhysicalProperties("Bulletproof");

                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Bottom";

                        if (Instance.GetComponent<ShieldBehaviour>() == null)
                        {
                            Instance.AddComponent(typeof(ShieldBehaviour));
                        }
                        var Door = Instance.GetComponent<ShieldBehaviour>();

                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 2f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 2f;
                        Instance.GetComponent<PhysicalBehaviour>().rigidbody.mass = 2f;
                    }
                }
            );
            //Ostrumian Exotic Engine
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Small I-Beam"),
                    NameOverride = "Ostrumian Exotic Engine -OF",
                    NameToOrderByOverride = "H",
                    DescriptionOverride = "An engine used in small starships that runs on Exotic Matter. Doesn't actually generate power. Does light up though.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/ExoticEngineView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ExoticEngine.png");
                        Instance.GetComponent<GlowingHotMetalBehaviour>().enabled = false;

                        var item = Instance.GetComponent<PhysicalBehaviour>();
                        item.Properties = ModAPI.FindPhysicalProperties("Insulator");
                        item.ForceNoChargeParticles = true;
                        Instance.FixColliders();
                        Instance.GetOrAddComponent<NoTemperature>();
                        if (Instance.GetComponent<AfterSpawnBehaviour>() == null)
                        {
                            Instance.AddComponent<AfterSpawnBehaviour>();
                            Instance.AddComponent<CustomGlowBattery>();
                        }

                        Instance.GetComponent<CustomGlowBattery>().TargetCharge = 0f;
                        Instance.GetComponent<CustomGlowBattery>().GlowSprite = ModAPI.LoadSprite("sprite/ExoticEngineMask.png");
                        Instance.GetComponent<CustomGlowBattery>().Colour = new Color32(50, 50, 200, 100);

                        Instance.GetComponent<PhysicalBehaviour>().HoldingPositions = new Vector3[0];
                    }
                }
            );
            //Ostrumian Magnetic Plate
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Small I-Beam"),
                    NameOverride = "Ostrumian Magnetic Plate -OF",
                    NameToOrderByOverride = "KKKKKKa",
                    DescriptionOverride = "Upon activation, sticks to just about anything. Even if they aren't magnetic. How does that work?",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/MagnetView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Magnet.png");
                        Instance.FixColliders();

                        if (Instance.GetComponent<AfterSpawnBehaviour>() == null)
                        {
                            Instance.AddComponent<AfterSpawnBehaviour>();

                            Instance.AddComponent<ClawBehaviour>();
                        }
                    }
                }
            );
            //Ostrumian White Rod
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Ostrumian White Rod -OF",
                    NameToOrderByOverride = "B",
                    DescriptionOverride = "A metal rod that has been painted white. Made of steel.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/WhiteRodView.png"),
                    AfterSpawn = (Instance) =>
                    {
                     Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/WhiteRod.png");
                    }
                }
            );
            //Ostrumian Black Rod
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Ostrumian Black Rod -OF",
                    NameToOrderByOverride = "BB",
                    DescriptionOverride = "A metal rod that has been painted black. Made of steel.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/BlackRodView.png"),
                    AfterSpawn = (Instance) =>
                    {
                     Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/BlackRod.png");
                    }
                }
            );
            //Ostrumian Graphene Reinforced Glass
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Ostrumian Graphene Reinforced Glass -OF",
                    NameToOrderByOverride = "BBBB",
                    DescriptionOverride = "Glass that has been reinforced with Graphene, and lined with Infragium. Doesn't shatter.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/GrapheneGlassView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/GrapheneGlass.png");
                        Instance.FixColliders();
                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Glass");
                    }
                }
            );
            //Ostrumian Background Rod
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Bulletproof Sheet"),
                    NameOverride = "Ostrumian Background Rod -OF",
                    NameToOrderByOverride = "BBB",
                    DescriptionOverride = "Use this to fill in areas where Ostrumians can't access / walk through. Disable collisions.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/BackgroundRodView.png"),
                    AfterSpawn = (Instance) =>
                    {
                     Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Bottom";
                     Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/BackgroundRod.png");
                    }
                }
            );
            //Ostrumian Weight
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("1000kg Weight"),
                    NameOverride = "Ostrumian Weight -OF",
                    NameToOrderByOverride = "C",
                    DescriptionOverride = "Heavy box.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/WeightView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                        {
                            ModAPI.LoadSprite("sprite/Weight.png"),

                        };
                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Android
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Android"),
                    NameOverride = "Ostrumian Android -OF",
                    NameToOrderByOverride = "Z",
                    DescriptionOverride = "Very durable Ostrumian-like robots. Aren't very capable in battle and are usually used in construction or other dangerous tasks.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/AndroidView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OuterShellLayer.png");
                        var flesh = ModAPI.LoadTexture("sprite/ElectronicsLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/MetalFrameLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);
                    }
                }
            );
            //Ostrumian Laser Pointer
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Laser Pointer"),
                    NameOverride = "Ostrumian Laser Pointer -OF",
                    NameToOrderByOverride = "KKKKKKKK",
                    DescriptionOverride = "A small device that fires a beam of photons.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/LaserPointerView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/LaserPointer.png");
                    }
                }
            );
            //Ostrumian Immobility Field
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Immobility Field"),
                    NameOverride = "Ostrumian Immobility Field Generator -OF",
                    NameToOrderByOverride = "KKKKKKK",
                    DescriptionOverride = "A device that generates a spherical field of gravitons that attempts to immobilise anything inside of it.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/ImmobilityFieldView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ImmobilityField.png");
                    }
                }
            );
            //Ostrumian Beam
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("I-Beam"),
                    NameOverride = "Ostrumian Beam -OF",
                    NameToOrderByOverride = "D",
                    DescriptionOverride = "A steel beam. Pretty heavy. Colored purple. Lined with Infragium, making it somewhat bulletproof.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/BeamView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                        {
                            ModAPI.LoadSprite("sprite/Beam.png"),
                        };
                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Bulletproof");
                    }
                }
            );
            //Ostrumian Small Beam
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Small I-Beam"),
                    NameOverride = "Small Ostrumian Beam -OF",
                    NameToOrderByOverride = "DD",
                    DescriptionOverride = "A purple steel beam that's been cut to be shorter. Lined with Infragium, making it somewhat bulletproof.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/SmallBeamView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/SmallBeam.png");
                        Instance.FixColliders();
                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Bulletproof");
                    }
                }
            );
            //Ostrumian Metal Pole
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Pole"),
                    NameOverride = "Ostrumian Metal Pole -OF",
                    NameToOrderByOverride = "DDD",
                    DescriptionOverride = "A long, thin rod of metal.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/MetalPoleView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/MetalPole.png");
                        Instance.FixColliders();
                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Bulletproof");
                    }
                }
            );
            //Ostrumian Flag
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Ostrumian Flag -OF",
                    NameToOrderByOverride = "F",
                    DescriptionOverride = "A flag of the Ostrumian Federation.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/FlagView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Flag.png");
                        Instance.FixColliders();
                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Soft");
                    }
                }
            );
            //Ostrumian Electron Bomb
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Energy Vessel Explosive"),
                    NameOverride = "Ostrumian Electron Bomb -OF",
                    NameToOrderByOverride = "OO",
                    DescriptionOverride = "A large and heavy device that creates a vortex using concentrated gravitons. It fires in a massive amount of electrons at the same time, and once the gravitons dissipate the immense pressure of electrons is released. Uses many materials, such as Gravitasium, Prolotosium, Transvorium, Aestusium and Obstructonium. Stupidly expensive.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/ElectronBombView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ElectronBomb.png");
                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Propeller
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Propeller"),
                    NameOverride = "Ostrumian Propeller -OF",
                    NameToOrderByOverride = "KKKKK",
                    DescriptionOverride = "Propels.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/PropellerView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Propeller.png");
                        Instance.transform.GetChild(0).GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/PropellerBlade.png");
                        Instance.transform.GetChild(1).GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/PropellerBlade.png");
                    }
                }
            );
            //Ostrumian Boat Motor
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Boat Motor"),
                    NameOverride = "Ostrumian Boat Motor -OF",
                    NameToOrderByOverride = "KKKKK",
                    DescriptionOverride = "Propels through water.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/BoatMotorView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/BoatMotor.png");
                    }
                }
            );
            //Ostrumian Dampening Box
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Dampening Box"),
                    NameOverride = "Ostrumian Dampening Box -OF",
                    NameToOrderByOverride = "KKKKKK",
                    DescriptionOverride = "Similar to the Immobility Field Generator, except the gravitons are localised within itself, causing it to slow down when activated.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/HoverboxView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Hoverbox.png");
                    }
                }
            );
            //Ostrumian Probe
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Dampening Box"),
                    NameOverride = "Ostrumian Probe -OF",
                    NameToOrderByOverride = "Y",
                    DescriptionOverride = "A long-distance space probe which has an AI equipped with a personality module, allowing it to act sentient and have conversations without actually being sentient. Used to find other intelligent life in the galaxy.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/ProbeView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Probe.png");
                        Instance.GetComponent<SpriteRenderer>().material = ModAPI.FindMaterial("Sprites-Default");

                        foreach (var c in Instance.GetComponents<Collider2D>())
                        {
                            GameObject.Destroy(c);
                        }
                        Instance.AddComponent<BoxCollider2D>();

                        if (Instance.GetComponent<AfterSpawnBehaviour>() == null)
                        {
                            Instance.AddComponent<AfterSpawnBehaviour>();
                            Instance.AddComponent<CustomGlowBattery>();
                        }

                        Instance.GetComponent<CustomGlowBattery>().TargetCharge = 0f;
                        Instance.GetComponent<CustomGlowBattery>().GlowSprite = ModAPI.LoadSprite("sprite/ProbeMask.png");
                            Instance.GetComponent<CustomGlowBattery>().Colour = new Color32(161, 102, 237, 100);

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.ForceNoChargeParticles = true;
                        phys.SimulateTemperature = false;
                        phys.Properties = ModAPI.FindPhysicalProperties("Metal");
                    }
                }
            );
            //Ostrumian Big Gyroscope
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Industrial Gyrostabiliser"),
                    NameOverride = "Industrial Ostrumian Gyrostabiliser -OF",
                    NameToOrderByOverride = "JJ",
                    DescriptionOverride = "A version of the gyrostabiliser for industrial use. A lot stronger than the other, smaller one.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/BigStabilizerView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/BigStabilizer.png");
                    }
                }
            );
            //Ostrumian Lagbox
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Lagbox"),
                    NameOverride = "Ostrumian Lagbox -OF",
                    NameToOrderByOverride = "QQQ",
                    DescriptionOverride = "Delays signals.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/LagboxView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Lagbox.png");
                        Instance.transform.GetChild(0).GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/LagboxKnob.png");
                    }
                }
            );
            //Ostrumian Gate
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Gate"),
                    NameOverride = "Ostrumian Gate -OF",
                    NameToOrderByOverride = "QQQ",
                    DescriptionOverride = "A device that only lets activation signals through if it is fed enough power.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/GateView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Gate.png");
                    }
                }
            );
            //Ostrumian Gyroscope
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Gyroscope Stabiliser"),
                    NameOverride = "Ostrumian Gyrostabiliser -OF",
                    NameToOrderByOverride = "J",
                    DescriptionOverride = "Uses reaction wheels activated in specific circumstances to attempt to make itself always upright.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/StabilizerView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Stabilizer.png");
                    }
                }
            );
            //Ostrumian Key Trigger
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Key Trigger"),
                    NameOverride = "Ostrumian Key Trigger -OF",
                    NameToOrderByOverride = "QQ",
                    DescriptionOverride = "Sends an activation signal depending on what key is pressed.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/KeyTriggerView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/KeyTrigger.png");
                    }
                }
            );
            //Ostrumian Metronome
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metronome"),
                    NameOverride = "Ostrumian Metronome -OF",
                    NameToOrderByOverride = "QQQQ",
                    DescriptionOverride = "A small device which sends out a signal every time it's hand hits either side. The speed is changeable.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/MetronomeView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Metronome.png");
                        Instance.transform.GetChild(0).GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/MetronomeHand.png");
                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Metal");
                    }
                }
            );
            //Ostrumian Hover Thruster
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Hover Thruster"),
                    NameOverride = "Ostrumian Hover Thruster -OF",
                    NameToOrderByOverride = "K",
                    DescriptionOverride = "Sends out a beam of gravitons which attempt to constantly keep it above the surface of whatever is under it.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/HoverThrusterView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/HoverThruster.png");
                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Radio
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Radio"),
                    NameOverride = "Ostrumian Radio -OF",
                    NameToOrderByOverride = "RR",
                    DescriptionOverride = "Sends an activation signal to any other radios on the same channel.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/RadioView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Radio.png");
                    }
                }
            );
            //Ostrumian Reconstructor
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Reconstructor"),
                    NameOverride = "Ostrumian Reconstructor -OF",
                    NameToOrderByOverride = "H",
                    DescriptionOverride = "Shoots out a mixture of bacteria and nanobots that attempt to stop bleeding and heal wounds. A temporary solution, so it is only really used on the battlefield.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/ReconstructorView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Reconstructor.png", 1f);
                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Laser Receiver
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Laser Receiver"),
                    NameOverride = "Ostrumian Laser Receiver -OF",
                    NameToOrderByOverride = "KKKKKKKKK",
                    DescriptionOverride = "When a laser hits it it sends an activation signal.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/LaserReceiverView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/LaserReceiver.png");
                    }
                }
            );
            //Ostrumian Contraurium Explosive
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Pink Explosive"),
                    NameOverride = "Ostrumian Contraurium Explosive -OF",
                    NameToOrderByOverride = "N",
                    DescriptionOverride = "An explosive which, when activated, releases a canister of pure Oxygen into a canister filled with Contraurium which causes a very violent reaction to occur.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/PinkExplosiveView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/PinkExplosive.png");
                    }
                }
            );
            //Ostrumian Plastic Explosive
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Plastic Explosive"),
                    NameOverride = "Ostrumian Plastic Explosive -OF",
                    NameToOrderByOverride = "M",
                    DescriptionOverride = "A powerful explosive device used for demolition.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/PlasticExplosiveView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/PlasticExplosive.png");
                    }
                }
            );
            //Ostrumian General Purpose Bomb
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("General Purpose Bomb"),
                    NameOverride = "Ostrumian General Purpose Bomb -OF",
                    NameToOrderByOverride = "O",
                    DescriptionOverride = "A very powerful explosive that is dropped from aircraft.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/GeneralPurposeBombView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/GeneralPurposeBomb.png");
                    }
                }
            );
            //Ostrumian Luxianium Bomb
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("EMP Generator"),
                    NameOverride = "Ostrumian Luxanium Bomb -OF",
                    NameToOrderByOverride = "O",
                    DescriptionOverride = "A device that utilizes Luxianium in order to generate an electromagnetic pulse. Most of the time, it breaks after a single use.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/EMPView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/EMP.png");
                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Text Display
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Text Display"),
                    NameOverride = "Ostrumian Text Display -OF",
                    NameToOrderByOverride = "RRR",
                    DescriptionOverride = "Displays text.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/DisplayView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/Display.png");
                    }
                }
            );
            //Ostrumian Structural Pillar
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Structural Pillar"),
                    NameOverride = "Ostrumian Structural Pillar -OF",
                    NameToOrderByOverride = "E",
                    DescriptionOverride = "A structural pillar used for reinforcement.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/StructuralPillarView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/StructuralPillar.png");
                        Instance.FixColliders();
                    }
                }
            );
            //Ostrumian Shock Detector
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Shock Detector"),
                    NameOverride = "Ostrumian Shock Detector -OF",
                    NameToOrderByOverride = "R",
                    DescriptionOverride = "Detects impact.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/ShockDetectorView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/ShockDetector.png");
                    }
                }
            );
            //Maid Dress Ostrumian
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Maid Dress Ostrumian -OF",
                    NameToOrderByOverride = "ZZZZZZZZ",
                    DescriptionOverride = "Congrats, you found the secret! This item is here because of an inside joke in the Arcadia Discord server about how all Ostrumians are, apparently, femboys.",
                    CategoryOverride = ModAPI.FindCategory("? -OF"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/BareOstrumianView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/ZZ.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        var person = Instance.GetComponent<PersonBehaviour>();
                        person.SetBodyTextures(skin, flesh, bone, 1);
                    }
                }
            );
            //Bare Ostrumian
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Bare Ostrumian -OF",
                    NameToOrderByOverride = "A",
                    DescriptionOverride = "An Ostrumian. They have 4 eyes and thick purple skin that protects them from the acidic rain on their planet. Their bones are actually a similar material to quartz.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/BareOstrumianView.png"),
                    AfterSpawn = (Instance) => 
                    {
                        var skin = ModAPI.LoadTexture("sprite/BareOstrumianSkinLayer.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        var person = Instance.GetComponent<PersonBehaviour>();
                        person.SetBodyTextures(skin, flesh, bone, 1);
                    }
                }
            );
            //Ostrumian Civilian 1
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Ostrumian Civilian 1 -OF",
                    NameToOrderByOverride = "B",
                    DescriptionOverride = "Average chad Ostrumian Civilian",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/Civ1View.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianCivilian1.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        var person = Instance.GetComponent<PersonBehaviour>();
                        person.SetBodyTextures(skin, flesh, bone, 1);
                    }
                }
            );
            //Ostrumian Civilian 2
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Ostrumian Civilian 2 -OF",
                    NameToOrderByOverride = "C",
                    DescriptionOverride = "Average chad Ostrumian Civilian",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/Civ2View.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianCivilian2.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        var person = Instance.GetComponent<PersonBehaviour>();
                        person.SetBodyTextures(skin, flesh, bone, 1);
                    }
                }
            );
            //Ostrumian Civilian 3
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Ostrumian Civilian 3 -OF",
                    NameToOrderByOverride = "CC",
                    DescriptionOverride = "Average chad Ostrumian Civilian",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/Civ1View.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianCivilian3.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        var person = Instance.GetComponent<PersonBehaviour>();
                        person.SetBodyTextures(skin, flesh, bone, 1);
                    }
                }
            );
            //Ostrumian Civilian 4
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Ostrumian Civilian 4 -OF",
                    NameToOrderByOverride = "CCC",
                    DescriptionOverride = "Average chad Ostrumian Civilian",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/Civ2View.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianCivilian4.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        var person = Instance.GetComponent<PersonBehaviour>();
                        person.SetBodyTextures(skin, flesh, bone, 1);
                    }
                }
            );
            //Ostrumian Labour Worker
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Ostrumian Labour Worker -OF",
                    NameToOrderByOverride = "D",
                    DescriptionOverride = "A general term for construction workers, warehouse workers, and manufacturing workers.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/LabourWorkerView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianLabourWorker.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        var person = Instance.GetComponent<PersonBehaviour>();
                        person.SetBodyTextures(skin, flesh, bone, 1);
                    }
                }
            );
            //Ostrumian Prisoner
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Ostrumian Prisoner -OF",
                    NameToOrderByOverride = "DD",
                    DescriptionOverride = "A criminal wearing a prison jumpsuit. ",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/PrisonerView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianPrisoner.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        var person = Instance.GetComponent<PersonBehaviour>();
                        person.SetBodyTextures(skin, flesh, bone, 1);
                    }
                }
            );
            //Ostrumian Scientist
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Ostrumian Scientist -OF",
                    NameToOrderByOverride = "E",
                    DescriptionOverride = "An Ostrumian scientist whos job it is to research into new technologies.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/ScientistView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianScientist.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);
                        var person = Instance.GetComponent<PersonBehaviour>();
                        //cosmetics
                        var head = Instance.transform.GetChild(5);
                        var childObject = new GameObject("Cape");
                        childObject.transform.SetParent(Instance.transform.Find("Body").Find("LowerBody"));
                        childObject.transform.localPosition = new Vector3(0, -0.0019f);
                        childObject.transform.rotation = Quaternion.Euler(0f, 0f, 0f);
                        childObject.transform.localScale = new Vector3(1f, 1f);
                        var childSprite = childObject.AddComponent<SpriteRenderer>();
                        childSprite.sprite = ModAPI.LoadSprite("sprite/ScientistCape.png");
                        childSprite.sortingLayerName = "Top";
                    }
                }
            );
            //Ostrumian Civilian EVA Suit
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Ostrumian Civilian EVA Suit -OF",
                    NameToOrderByOverride = "F",
                    DescriptionOverride = "A civilian wearing a standard personal EVA Suit.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/CivEVAView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/CivilianEVASuit.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.gameObject.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                            body.BaseStrength = 12f;
                            body.Health = 50f;
                            body.InitialHealth = 50f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 20f;
                            body.CirculationBehaviour.ImmuneToDamage = true;
                            body.gameObject.AddComponent<Strength>();
                        }
                        //cosmetics
                        var head = Instance.transform.GetChild(5);
                        var childObject = new GameObject("Backpack");
                        childObject.transform.SetParent(Instance.transform.Find("Body").Find("UpperBody"));
                        childObject.transform.localPosition = new Vector3(0, 0f);
                        childObject.transform.rotation = Quaternion.Euler(0f, 0f, 0f);
                        childObject.transform.localScale = new Vector3(1f, 1f);
                        var childSprite = childObject.AddComponent<SpriteRenderer>();
                        childSprite.sprite = ModAPI.LoadSprite("sprite/CivilianBackpack.png");
                    }
                }
            );
            //Ostrumian Sailor
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Ostrumian Sailor -OF",
                    NameToOrderByOverride = "GG",
                    DescriptionOverride = "An person in the navy wearing a standard EVA suit.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/NavyView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianNavy.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.gameObject.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                            body.BaseStrength = 12f;
                            body.Health = 60f;
                            body.InitialHealth = 60f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 30f;
                            body.CirculationBehaviour.ImmuneToDamage = true;
                            body.gameObject.AddComponent<Strength>();
                        }
                        //cosmetics
                        var head = Instance.transform.GetChild(5);
                        var childObject = new GameObject("Backpack");
                        childObject.transform.SetParent(Instance.transform.Find("Body").Find("UpperBody"));
                        childObject.transform.localPosition = new Vector3(0, 0f);
                        childObject.transform.rotation = Quaternion.Euler(0f, 0f, 0f);
                        childObject.transform.localScale = new Vector3(1f, 1f);
                        var childSprite = childObject.AddComponent<SpriteRenderer>();
                        childSprite.sprite = ModAPI.LoadSprite("sprite/NavyBackpack.png");
                    }
                }
            );
            //Ostrumian Miner EVA Suit
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Ostrumian Miner -OF",
                    NameToOrderByOverride = "G",
                    DescriptionOverride = "A miner wearing a standard issue EVA Suit. They use a Handheld Mining Drill.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/MinerView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianMiner.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.gameObject.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                            body.BaseStrength = 12f;
                            body.Health = 50f;
                            body.InitialHealth = 50f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 20f;
                            body.CirculationBehaviour.ImmuneToDamage = true;
                            body.gameObject.AddComponent<Strength>();
                        }
                        //cosmetics
                        var head = Instance.transform.GetChild(5);
                        var childObject = new GameObject("Backpack");
                        childObject.transform.SetParent(Instance.transform.Find("Body").Find("UpperBody"));
                        childObject.transform.localPosition = new Vector3(0, 0f);
                        childObject.transform.rotation = Quaternion.Euler(0f, 0f, 0f);
                        childObject.transform.localScale = new Vector3(1f, 1f);
                        var childSprite = childObject.AddComponent<SpriteRenderer>();
                        childSprite.sprite = ModAPI.LoadSprite("sprite/MinerBackpack.png");
                    }
                }
            );
            //Ostrumian Scientist EVA Suit
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Ostrumian Scientist EVA Suit -OF",
                    NameToOrderByOverride = "GG",
                    DescriptionOverride = "A scientist wearing a standard issue EVA Suit.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/ScientistEVAView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianScientistEVA.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.gameObject.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                            body.BaseStrength = 12f;
                            body.Health = 50f;
                            body.InitialHealth = 50f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 20f;
                            body.CirculationBehaviour.ImmuneToDamage = true;
                            body.gameObject.AddComponent<Strength>();
                        }
                        //cosmetics
                        var head = Instance.transform.GetChild(5);
                        var childObject = new GameObject("Backpack");
                        childObject.transform.SetParent(Instance.transform.Find("Body").Find("UpperBody"));
                        childObject.transform.localPosition = new Vector3(0, 0f);
                        childObject.transform.rotation = Quaternion.Euler(0f, 0f, 0f);
                        childObject.transform.localScale = new Vector3(1f, 1f);
                        var childSprite = childObject.AddComponent<SpriteRenderer>();
                        childSprite.sprite = ModAPI.LoadSprite("sprite/ScientistEVABackpack.png");
                    }
                }
            );
            //Ostrumian Peacekeeper
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Ostrumian Peacekeeper -OF",
                    NameToOrderByOverride = "H",
                    DescriptionOverride = "The police force of the Ostrumian Federation.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/PoliceView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianPolice.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.gameObject.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                            body.BaseStrength = 12f;
                            body.Health = 55f;
                            body.InitialHealth = 55f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 25f;
                            body.CirculationBehaviour.ImmuneToDamage = true;
                            body.gameObject.AddComponent<Strength>();
                        }
                        //cosmetics
                        var head = Instance.transform.GetChild(5);
                        var childObject = new GameObject("Backpack");
                        childObject.transform.SetParent(Instance.transform.Find("Body").Find("UpperBody"));
                        childObject.transform.localPosition = new Vector3(0, 0f);
                        childObject.transform.rotation = Quaternion.Euler(0f, 0f, 0f);
                        childObject.transform.localScale = new Vector3(1f, 1f);
                        var childSprite = childObject.AddComponent<SpriteRenderer>();
                        childSprite.sprite = ModAPI.LoadSprite("sprite/PoliceBackpack.png");
                    }
                }
            );
            //Ostrumian Soldier
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Ostrumian Soldier -OF",
                    NameToOrderByOverride = "H",
                    DescriptionOverride = "Average soldier in the Ostrumian Federation. Uses a rifle or shotgun.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/Soldier1View.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianSoldier1.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.gameObject.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                            body.BaseStrength = 12f;
                            body.Health = 60f;
                            body.InitialHealth = 60f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 30f;
                            body.CirculationBehaviour.ImmuneToDamage = true;
                            body.gameObject.AddComponent<Strength>();
                        }
                        //cosmetics
                        var head = Instance.transform.GetChild(5);
                        var childObject = new GameObject("Backpack");
                        childObject.transform.SetParent(Instance.transform.Find("Body").Find("UpperBody"));
                        childObject.transform.localPosition = new Vector3(0, 0f);
                        childObject.transform.rotation = Quaternion.Euler(0f, 0f, 0f);
                        childObject.transform.localScale = new Vector3(1f, 1f);
                        var childSprite = childObject.AddComponent<SpriteRenderer>();
                        childSprite.sprite = ModAPI.LoadSprite("sprite/SoldierBackpack1.png");
                    }
                }
            );
            //Ostrumian Thermal Soldier
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Ostrumian Thermal Soldier -OF",
                    NameToOrderByOverride = "L",
                    DescriptionOverride = "A soldier with much more protection against higher temperatures. Uses a flamethrower.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/ThermalSoldierView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianThermalSoldier.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.gameObject.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                            body.BaseStrength = 12f;
                            body.Health = 60f;
                            body.InitialHealth = 60f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 30f;
                            body.CirculationBehaviour.ImmuneToDamage = true;
                            body.gameObject.AddComponent<Strength>();
                        }
                        //cosmetics
                        var head = Instance.transform.GetChild(5);
                        var childObject = new GameObject("Backpack");
                        childObject.transform.SetParent(Instance.transform.Find("Body").Find("UpperBody"));
                        childObject.transform.localPosition = new Vector3(0, 0f);
                        childObject.transform.rotation = Quaternion.Euler(0f, 0f, 0f);
                        childObject.transform.localScale = new Vector3(1f, 1f);
                        var childSprite = childObject.AddComponent<SpriteRenderer>();
                        childSprite.sprite = ModAPI.LoadSprite("sprite/ThermalSoldierBackpack.png");
                    }
                }
            );
            //Ostrumian Captain
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Ostrumian Captain -OF",
                    NameToOrderByOverride = "N",
                    DescriptionOverride = "A high ranking Ostrumian in the army. Leads troops.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/CommanderView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianCommander.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.gameObject.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                            body.BaseStrength = 14f;
                            body.Health = 70f;
                            body.InitialHealth = 70f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 50f;
                            body.CirculationBehaviour.ImmuneToDamage = true;
                            body.gameObject.AddComponent<Strength>();
                            body.PhysicalBehaviour.ForceNoCharge = true;
                        }
                        //cosmetics
                        var head = Instance.transform.GetChild(5);
                        var childObject = new GameObject("Backpack");
                        childObject.transform.SetParent(Instance.transform.Find("Body").Find("UpperBody"));
                        childObject.transform.localPosition = new Vector3(0, 0f);
                        childObject.transform.rotation = Quaternion.Euler(0f, 0f, 0f);
                        childObject.transform.localScale = new Vector3(1f, 1f);
                        var childSprite = childObject.AddComponent<SpriteRenderer>();
                        childSprite.sprite = ModAPI.LoadSprite("sprite/CommanderBackpack.png");
                    }
                }
            );
            //Ostrumian Demolitions Expert
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Ostrumian Demolitions Expert -OF",
                    NameToOrderByOverride = "M",
                    DescriptionOverride = "A millitary demolitions expert. Usually uses a grenade launcher.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/DemomanView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianDemoman.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.gameObject.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                            body.BaseStrength = 13f;
                            body.Health = 55f;
                            body.InitialHealth = 55f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 30f;
                            body.CirculationBehaviour.ImmuneToDamage = true;
                            body.gameObject.AddComponent<Strength>();
                        }
                        //cosmetics
                        var head = Instance.transform.GetChild(5);
                        var childObject = new GameObject("Backpack");
                        childObject.transform.SetParent(Instance.transform.Find("Body").Find("UpperBody"));
                        childObject.transform.localPosition = new Vector3(0, 0f);
                        childObject.transform.rotation = Quaternion.Euler(0f, 0f, 0f);
                        childObject.transform.localScale = new Vector3(1f, 1f);
                        var childSprite = childObject.AddComponent<SpriteRenderer>();
                        childSprite.sprite = ModAPI.LoadSprite("sprite/DemoBackpack.png");
                    }
                }
            );
            //Ostrumian Pilot
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Ostrumian Pilot -OF",
                    NameToOrderByOverride = "I",
                    DescriptionOverride = "A millitary pilot. They fly aircraft, obviously. What were you expecting?",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/PilotView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianPilot.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.gameObject.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                            body.BaseStrength = 12f;
                            body.Health = 50f;
                            body.InitialHealth = 50f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 30f;
                            body.CirculationBehaviour.ImmuneToDamage = true;
                            body.gameObject.AddComponent<Strength>();
                        }
                        //cosmetics
                        var head = Instance.transform.GetChild(5);
                        var childObject = new GameObject("Backpack");
                        childObject.transform.SetParent(Instance.transform.Find("Body").Find("UpperBody"));
                        childObject.transform.localPosition = new Vector3(0, 0f);
                        childObject.transform.rotation = Quaternion.Euler(0f, 0f, 0f);
                        childObject.transform.localScale = new Vector3(1f, 1f);
                        var childSprite = childObject.AddComponent<SpriteRenderer>();
                        childSprite.sprite = ModAPI.LoadSprite("sprite/PilotBackpack.png");
                    }
                }
            );
            //Ostrumian Medic
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Ostrumian Medic -OF",
                    NameToOrderByOverride = "J",
                    DescriptionOverride = "A millitary Medic. They help injured people. Always carrying a reconstructor.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/MedicView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianMedic.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.gameObject.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                            body.BaseStrength = 12f;
                            body.Health = 50f;
                            body.InitialHealth = 50f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 25f;
                            body.CirculationBehaviour.ImmuneToDamage = true;
                            body.gameObject.AddComponent<Strength>();
                        }
                        //cosmetics
                        var head = Instance.transform.GetChild(5);
                        var childObject = new GameObject("Backpack");
                        childObject.transform.SetParent(Instance.transform.Find("Body").Find("UpperBody"));
                        childObject.transform.localPosition = new Vector3(0, 0f);
                        childObject.transform.rotation = Quaternion.Euler(0f, 0f, 0f);
                        childObject.transform.localScale = new Vector3(1f, 1f);
                        var childSprite = childObject.AddComponent<SpriteRenderer>();
                        childSprite.sprite = ModAPI.LoadSprite("sprite/MedicBackpack.png");
                    }
                }
            );
            //Ostrumian Heavy Soldier
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Ostrumian Heavy Soldier -OF",
                    NameToOrderByOverride = "O",
                    DescriptionOverride = "A heavy soldier. They either use a shotgun, rifle or light machine gun.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/HeavySoldierView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianHeavySoldier.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/HeavyBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.gameObject.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                            body.BaseStrength = 18f;
                            body.Health = 90f;
                            body.InitialHealth = 90f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 80f;
                            body.CirculationBehaviour.ImmuneToDamage = true;
                            body.gameObject.AddComponent<Strength>();
                            body.PhysicalBehaviour.ForceNoCharge = true;
                        }
                        //cosmetics
                        var head = Instance.transform.GetChild(5);
                        var childObject = new GameObject("Backpack");
                        childObject.transform.SetParent(Instance.transform.Find("Body").Find("UpperBody"));
                        childObject.transform.localPosition = new Vector3(0, 0f);
                        childObject.transform.rotation = Quaternion.Euler(0f, 0f, 0f);
                        childObject.transform.localScale = new Vector3(1f, 1f);
                        var childSprite = childObject.AddComponent<SpriteRenderer>();
                        childSprite.sprite = ModAPI.LoadSprite("sprite/HeavySoldierBackpack.png");
                    }
                }
            );
            //Ostrumian Sniper
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Ostrumian Sniper -OF",
                    NameToOrderByOverride = "K",
                    DescriptionOverride = "A Sniper. They use a sniper rifle (duh).",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/SniperView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianSniper.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.gameObject.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                            body.BaseStrength = 15f;
                            body.Health = 70f;
                            body.InitialHealth = 70f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 70f;
                            body.CirculationBehaviour.ImmuneToDamage = true;
                            body.gameObject.AddComponent<Strength>();
                        }
                        //cosmetics
                        var head = Instance.transform.GetChild(5);
                        var childObject = new GameObject("Backpack");
                        childObject.transform.SetParent(Instance.transform.Find("Body").Find("UpperBody"));
                        childObject.transform.localPosition = new Vector3(0, 0f);
                        childObject.transform.rotation = Quaternion.Euler(0f, 0f, 0f);
                        childObject.transform.localScale = new Vector3(1f, 1f);
                        var childSprite = childObject.AddComponent<SpriteRenderer>();
                        childSprite.sprite = ModAPI.LoadSprite("sprite/SniperBackpack.png");
                    }
                }
            );
            //Ostrumian Energy Soldier
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Ostrumian Energy Soldier -OF",
                    NameToOrderByOverride = "Oo",
                    DescriptionOverride = "A specialised soldier that focuses on the use of energy weapons. Carries an Exotic Rifle.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/EnergySoldierView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianEnergySoldier.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/EnergyBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.gameObject.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                            body.BaseStrength = 17f;
                            body.Health = 65f;
                            body.InitialHealth = 65f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 80f;
                            body.CirculationBehaviour.ImmuneToDamage = true;
                            body.gameObject.AddComponent<Strength>();
                            body.PhysicalBehaviour.ForceNoCharge = true;
                        }
                        //cosmetics
                        var head = Instance.transform.GetChild(5);
                        var childObject = new GameObject("Backpack");
                        childObject.transform.SetParent(Instance.transform.Find("Body").Find("UpperBody"));
                        childObject.transform.localPosition = new Vector3(0, 0f);
                        childObject.transform.rotation = Quaternion.Euler(0f, 0f, 0f);
                        childObject.transform.localScale = new Vector3(1f, 1f);
                        var childSprite = childObject.AddComponent<SpriteRenderer>();
                        childSprite.sprite = ModAPI.LoadSprite("sprite/EnergySoldierBackpack.png");
                    }
                }
            );
            //Ostrumian Void Division Soldier
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Void Division Soldier -OF",
                    NameToOrderByOverride = "Z",
                    DescriptionOverride = "A soldier part of the Void Division. These soldiers are the best of the best in the Ostrumian military.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/VoidDivSoldierView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianVoidDivisionSoldier.png");
                        var flesh = ModAPI.LoadTexture("sprite/VoidFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/VoidBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.gameObject.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Bulletproof");
                            body.BaseStrength = 25f;
                            body.Health = 300f;
                            body.InitialHealth = 300f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 100f;
                            body.CirculationBehaviour.ImmuneToDamage = true;
                            body.gameObject.AddComponent<Strength>();
                            body.PhysicalBehaviour.ForceNoCharge = true;
                        }
                        //cosmetics
                        var head = Instance.transform.GetChild(5);
                        var childObject = new GameObject("Backpack");
                        childObject.transform.SetParent(Instance.transform.Find("Body").Find("UpperBody"));
                        childObject.transform.localPosition = new Vector3(0, 0f);
                        childObject.transform.rotation = Quaternion.Euler(0f, 0f, 0f);
                        childObject.transform.localScale = new Vector3(1f, 1f);
                        var childSprite = childObject.AddComponent<SpriteRenderer>();
                        childSprite.sprite = ModAPI.LoadSprite("sprite/VoidDivSoldierBackpack.png");
                    }
                }
            );
            //Ostrumian Void Division Sniper
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Void Division Sniper -OF",
                    NameToOrderByOverride = "Z",
                    DescriptionOverride = "A sniper part of the Void Division. These snipers are some of the greatest marksmen of the entire Ostrumian military.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/VoidDivSniperView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianVoidDivisionSniper.png");
                        var flesh = ModAPI.LoadTexture("sprite/VoidFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/VoidBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.gameObject.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Bulletproof");
                            body.BaseStrength = 25f;
                            body.Health = 300f;
                            body.InitialHealth = 300f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 100f;
                            body.CirculationBehaviour.ImmuneToDamage = true;
                            body.gameObject.AddComponent<Strength>();
                            body.PhysicalBehaviour.ForceNoCharge = true;
                        }
                        //cosmetics
                        var head = Instance.transform.GetChild(5);
                        var childObject = new GameObject("Backpack");
                        childObject.transform.SetParent(Instance.transform.Find("Body").Find("UpperBody"));
                        childObject.transform.localPosition = new Vector3(0, 0f);
                        childObject.transform.rotation = Quaternion.Euler(0f, 0f, 0f);
                        childObject.transform.localScale = new Vector3(1f, 1f);
                        var childSprite = childObject.AddComponent<SpriteRenderer>();
                        childSprite.sprite = ModAPI.LoadSprite("sprite/VoidDivSniperBackpack.png");
                    }
                }
            );
            //Ostrumian Cybersoldier
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Ostrumian Cybersoldier -OF",
                    NameToOrderByOverride = "Ya",
                    DescriptionOverride = "A soldier specialized to be fitted with large amounts of augmentations, to the point where they're more machine than Ostrumian.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/CyberSoldierView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianCyberSoldier.png");
                        var flesh = ModAPI.LoadTexture("sprite/CyberFlesh.png");
                        var bone = ModAPI.LoadTexture("sprite/CyberSkeleton.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        person.SetBodyTextures(skin, null, null, 1);
                        person.SetBruiseColor(50, 50, 50);
                        person.SetSecondBruiseColor(100, 100, 100);
                        person.SetThirdBruiseColor(150, 150, 150);

                        foreach (var body in person.Limbs)
                        {
                            body.gameObject.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Bulletproof");
                            body.BaseStrength = 25f;
                            body.Health = 500f;
                            body.InitialHealth = 500f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 250f;
                            body.CirculationBehaviour.ImmuneToDamage = true;
                            body.gameObject.AddComponent<Strength>();
                            body.gameObject.AddComponent<EMPDeath>();
                            body.PhysicalBehaviour.ForceNoCharge = false;
                        }
                        //cosmetics
                        var head = Instance.transform.GetChild(5);
                        var childObject = new GameObject("Backpack");
                        childObject.transform.SetParent(Instance.transform.Find("Body").Find("UpperBody"));
                        childObject.transform.localPosition = new Vector3(0, 0f);
                        childObject.transform.rotation = Quaternion.Euler(0f, 0f, 0f);
                        childObject.transform.localScale = new Vector3(1f, 1f);
                        var childSprite = childObject.AddComponent<SpriteRenderer>();
                        childSprite.sprite = ModAPI.LoadSprite("sprite/CyberBackpack.png");
                    }
                }
            );
            //Ostrumian Gravimancer
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Ostrumian Gravimancer -OF",
                    NameToOrderByOverride = "Yb",
                    DescriptionOverride = "A soldier with a specialised gravity-manipulating EVA pack.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/GravimancerView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/OstrumianGravimancer.png");
                        var flesh = ModAPI.LoadTexture("sprite/VoidFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/GravimancerSkeleton.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.gameObject.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Bulletproof");
                            body.BaseStrength = 15f;
                            body.Health = 250f;
                            body.InitialHealth = 250f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 100f;
                            body.CirculationBehaviour.ImmuneToDamage = true;
                            body.gameObject.AddComponent<Strength>();
                            body.PhysicalBehaviour.ForceNoCharge = false;
                            body.gameObject.GetComponent<PhysicalBehaviour>().rigidbody.gravityScale *= 0.5f;
                        }
                        //cosmetics
                        var head = Instance.transform.GetChild(5);
                        var childObject = new GameObject("Backpack");
                        childObject.transform.SetParent(Instance.transform.Find("Body").Find("UpperBody"));
                        childObject.transform.localPosition = new Vector3(0, 0f);
                        childObject.transform.rotation = Quaternion.Euler(0f, 0f, 0f);
                        childObject.transform.localScale = new Vector3(1f, 1f);
                        var childSprite = childObject.AddComponent<SpriteRenderer>();
                        childSprite.sprite = ModAPI.LoadSprite("sprite/GravimancerBackpack.png");
                    }
                }
            );
            //Sebastian Manus
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Sebastian Manus -OF",
                    NameToOrderByOverride = "ZZZZZZZZZZZZZZ",
                    DescriptionOverride = "The man, the myth, the legend. This is the engineer who; created the Combat Jim, accidentally almost caused an AI uprising which started from a toaster (don't ask), is a known alchaholic and schizopreniac, and has an IQ of 420.",
                    CategoryOverride = ModAPI.FindCategory("? -OF"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/BareOstrumianView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/SebastianManus.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.gameObject.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Bulletproof");
                            body.BaseStrength = 20f;
                            body.Health = 1000f;
                            body.InitialHealth = 1000f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 30f;
                            body.CirculationBehaviour.ImmuneToDamage = true;
                            body.gameObject.AddComponent<Strength>();
                            body.PhysicalBehaviour.ForceNoCharge = true;
                        }
                        //cosmetics
                        var head = Instance.transform.GetChild(5);
                        var childObject = new GameObject("Backpack");
                        childObject.transform.SetParent(Instance.transform.Find("Body").Find("UpperBody"));
                        childObject.transform.localPosition = new Vector3(0, 0f);
                        childObject.transform.rotation = Quaternion.Euler(0f, 0f, 0f);
                        childObject.transform.localScale = new Vector3(1f, 1f);
                        var childSprite = childObject.AddComponent<SpriteRenderer>();
                        childSprite.sprite = ModAPI.LoadSprite("sprite/SebastianManusBackpack.png");
                    }
                }
            );
            //Sebastian Manus Welding Mask
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Sebastian Manus' Welding Mask -OF",
                    NameToOrderByOverride = "ZZZZZZZZZZZ",
                    DescriptionOverride = "Sebastian Manus' Welding Mask.",
                    CategoryOverride = ModAPI.FindCategory("? -OF"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/CivHelmetView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/WeldingMask.png");

                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();

                        armor.stabResistance = 22f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 12;

                        var item = Instance.GetComponent<PhysicalBehaviour>();
                        item.ForceNoCharge = true;
                        item.Properties = ModAPI.FindPhysicalProperties("Insulator");
                        Instance.GetComponent<GlowingHotMetalBehaviour>().enabled = false;

                        Instance.FixColliders();
                    }
                }
            );
            //Aegis J. Dux
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "President Aegis J. Dux -OF",
                    NameToOrderByOverride = "ZZZa",
                    DescriptionOverride = "The current president of the Ostrumian Federation. Aegis Dux is known for being very charismatic, and won his most recent campaign with over 67% of votes.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/AegisDuxView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/AegisDux.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.BaseStrength = 12f;
                            body.Health = 100f;
                            body.InitialHealth = 100f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 40f;
                        }
                    }
                }
            );
            //Aegis J. Dux Hair
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Aegis J. Dux's Hair -OF",
                    NameToOrderByOverride = "ZZZZZZZZZZZ",
                    DescriptionOverride = "Aegis J. Dux's Hair.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/AegisDuxHairView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/AegisDuxHair.png");
                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();
                        armor.threepieces = false;

                        armor.stabResistance = 44f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 44;

                        var item = Instance.GetComponent<PhysicalBehaviour>();
                        item.ForceNoCharge = true;
                        item.Properties = ModAPI.FindPhysicalProperties("Soft");
                        Instance.GetComponent<GlowingHotMetalBehaviour>().enabled = false;

                        Instance.FixColliders();
                    }
                }
            );
            //Damien U. Scire
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Councillor Damien U. Scire -OF",
                    NameToOrderByOverride = "ZZZZ",
                    DescriptionOverride = "The oldest of the current 7 council members, being over 130 years old. Despite his age, he does his job incredibly well and has been re-elected for his role 5 times now. His body is heavily augmented, and always votes in favor of subjects which may lead to potential breakthroughs in science. He often gets a bit too ahead of himself however, and often ignores the potential danger and sacrifices that lead to these breakthroughs.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/DamienScireView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/DamienScire.png");
                        var flesh = ModAPI.LoadTexture("sprite/DamienScireFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/DamienScireBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.gameObject.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Bulletproof");
                            body.BaseStrength = 12f;
                            body.Health = 200f;
                            body.InitialHealth = 200f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 50f;
                        }
                    }
                }
            );
            //Gabriel S. Vis
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Councillor Gabriel S. Vis -OF",
                    NameToOrderByOverride = "ZZZZZ",
                    DescriptionOverride = "A purist, having no mechanical augmentations whatsoever, and a loyal follower of Electi Universi. He is often criticized for his rather extreme practices when it comes to Electi Universi rituals, and he often preaches to others to take up these practices as well.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/GabrielVisView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/GabrielVis.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.BaseStrength = 12f;
                            body.Health = 100f;
                            body.InitialHealth = 100f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 40f;
                        }
                    }
                }
            );
            //Gabriel S. Vis Hair
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Gabriel S. Vis' Hair -OF",
                    NameToOrderByOverride = "ZZZZZZZZZZZa",
                    DescriptionOverride = "Gabriel S. Vis' Hair.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/GabrielVisHairView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/GabrielVisHair.png");
                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();
                        armor.threepieces = false;

                        armor.stabResistance = 44f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 44;

                        var item = Instance.GetComponent<PhysicalBehaviour>();
                        item.ForceNoCharge = true;
                        item.Properties = ModAPI.FindPhysicalProperties("Soft");
                        Instance.GetComponent<GlowingHotMetalBehaviour>().enabled = false;

                        Instance.FixColliders();
                    }
                }
            );
            //Peter A. Invitus
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Councillor Peter A. Invitus -OF",
                    NameToOrderByOverride = "ZZZZZZ",
                    DescriptionOverride = "A mysterious council member. Not much is known about their past, but they are damn good at their job. They fight for equality and peace, and is a great leader. They listen to the requests of the people, and so far nobody has ever protested his leadership. This man has already been re-elected once, and further down the line in his career it is expected he will be re-elected again.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/PeterInvitusView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/PeterInvitus.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/PeterInvitusBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.BaseStrength = 12f;
                            body.Health = 100f;
                            body.InitialHealth = 100f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 45f;
                        }
                    }
                }
            );
            //Peter A. Invitus Hair
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Peter A. Invitus' Hair -OF",
                    NameToOrderByOverride = "ZZZZZZZZZZZb",
                    DescriptionOverride = "Peter A. Invitus' Hair.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/PeterInvitusHairView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/PeterInvitusHair.png");
                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();
                        armor.threepieces = false;

                        armor.stabResistance = 44f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 44;

                        var item = Instance.GetComponent<PhysicalBehaviour>();
                        item.ForceNoCharge = true;
                        item.Properties = ModAPI.FindPhysicalProperties("Soft");
                        Instance.GetComponent<GlowingHotMetalBehaviour>().enabled = false;

                        Instance.FixColliders();
                    }
                }
            );
            //Johnathon N. Scobo
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Councillor Johnathon N. Scobo -OF",
                    NameToOrderByOverride = "ZZZZZZ",
                    DescriptionOverride = "An incredibly eccentric and intelligent engineer and politician. He is known for being very charismatic and humorous. He is contantly working on new engineering projects, and is the founder of AtmoTec.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/JohnathonScoboView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/JohnathonScobo.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/JohnathonScoboBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.BaseStrength = 12f;
                            body.Health = 150f;
                            body.InitialHealth = 150f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 40f;
                        }
                    }
                }
            );
            //Fleur O. Luna
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Councillor Fleur O. Luna -OF",
                    NameToOrderByOverride = "ZZZZZZZ",
                    DescriptionOverride = "An ambitious councillor who is incredibly passionate about the colonization and research of exoplanets. She tends to get along well with Damien Scire, and when it comes to votes it's either both of them agreeing or neither of them agreeing. She has no augmentation whatsoever.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/FleurLunaView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/FleurLuna.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.BaseStrength = 12f;
                            body.Health = 100f;
                            body.InitialHealth = 100f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 40f;
                        }
                    }
                }
            );
            //Fleur O. Luna Hair
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Fleur O. Luna's Hair -OF",
                    NameToOrderByOverride = "ZZZZZZZZZZZZ",
                    DescriptionOverride = "Fleur O. Luna's Hair.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/FleurLunaHairView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/FleurLunaHair.png");
                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();
                        armor.threepieces = false;

                        armor.stabResistance = 44f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 44;

                        var item = Instance.GetComponent<PhysicalBehaviour>();
                        item.ForceNoCharge = true;
                        item.Properties = ModAPI.FindPhysicalProperties("Soft");
                        Instance.GetComponent<GlowingHotMetalBehaviour>().enabled = false;

                        Instance.FixColliders();
                    }
                }
            );
            //Iris K. Fulmen
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Android"),
                    NameOverride = "Councillor Iris K. Fulmen -OF",
                    NameToOrderByOverride = "ZZZZZZZZZZZZ",
                    DescriptionOverride = "A sentient android, and good politician. She was once an Ostrumian, but her consciousness was uploaded into a mechanical frame. Whether or not her mind was actually uploaded or copied is unknown. She is a great strategist and is involved in many military operations.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/IrisFulmenView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/IrisFulmen.png");
                        var flesh = ModAPI.LoadTexture("sprite/ElectronicsLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/MetalFrameLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);
                    }
                }
            );
            //Anthea V. Niveus
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"),
                    NameOverride = "Councillor Anthea V. Niveus -OF",
                    NameToOrderByOverride = "ZZZZZZZZZZZZZ",
                    DescriptionOverride = "This councillor was born with a kind of albinism, resulting in their pale and almost white skin. She is the youngest of the councillors, but is quite popular despite the small amout of time she has spent in the council. She is an environmentalist, and argues greatly against things which may result in damage against the environment.",
                    CategoryOverride = ModAPI.FindCategory("OF - Ostrumians"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/AntheaNiveusView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var skin = ModAPI.LoadTexture("sprite/AntheaNiveus.png");
                        var flesh = ModAPI.LoadTexture("sprite/OstrumianFleshLayer.png");
                        var bone = ModAPI.LoadTexture("sprite/OstrumianBoneLayer.png");
                        Instance.GetComponent<PersonBehaviour>().SetBodyTextures(skin, flesh, bone);

                        var person = Instance.GetComponent<PersonBehaviour>();

                        foreach (var body in person.Limbs)
                        {
                            body.BaseStrength = 12f;
                            body.Health = 100f;
                            body.InitialHealth = 100f;
                            body.ShotDamageMultiplier = 0f;
                            body.ImpactPainMultiplier = 0f;
                            body.CirculationBehaviour.BloodFlow = 40f;
                            body.BreakingThreshold = 40f;
                        }
                    }
                }
            );
            //Anthea V. Niveus Hair
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Anthea V. Niveus' Hair -OF",
                    NameToOrderByOverride = "ZZZZZZZZZZZZZ",
                    DescriptionOverride = "Anthea V. Niveus' Hair.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/AntheaNiveusHairView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/AntheaNiveusHair.png");
                        ArmorBehaviour armor = Instance.AddComponent<ArmorBehaviour>();
                        armor.threepieces = false;

                        armor.stabResistance = 44f;
                        armor.armorPiece = "Head";
                        armor.armorTier = 44;

                        var item = Instance.GetComponent<PhysicalBehaviour>();
                        item.ForceNoCharge = true;
                        item.Properties = ModAPI.FindPhysicalProperties("Soft");
                        Instance.GetComponent<GlowingHotMetalBehaviour>().enabled = false;

                        Instance.FixColliders();
                    }
                }
            );
            //ASR-559
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Sniper Rifle"),
                    NameOverride = "ASR-559 -OF",
                    NameToOrderByOverride = "TTTTTT",
                    DescriptionOverride = "An experimental yet incredibly powerful sniper rifle used by Void Division snipers.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/HandHeldRailgunView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/HandHeldRailgun.png");
                        var firearm = Instance.GetComponent<FirearmBehaviour>();
                        firearm.barrelPosition = new Vector3(0.825f, 0.115f);

                        Cartridge customCartridge = ModAPI.FindCartridge("30 mm");
                        customCartridge.name = "ASR-559 cartridge";
                        customCartridge.Damage *= 125f;
                        customCartridge.StartSpeed *= 169f;
                        customCartridge.PenetrationRandomAngleMultiplier *= 0f;
                        customCartridge.Recoil *= 0.2f;
                        customCartridge.ImpactForce *= 0.65f;
                        firearm.InitialInaccuracy *= 0f;
                        firearm.BulletsPerShot *= 1;
                        firearm.Cartridge = customCartridge;
                        firearm.ShotSounds = new AudioClip[]
                        {
                            ModAPI.LoadSound("railgunshot.wav"),
                        };
                        Instance.FixColliders();

                        Instance.AddComponent<UseEventTrigger>().Action = (System.Action)(() => ModAPI.CreateParticleEffect("Vapor", Instance.GetComponent<CanShoot>().BarrelPosition));
                    }
                }
            );
            //FS-007
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rifle"),
                    NameOverride = "FS-007 -OF",
                    NameToOrderByOverride = "TTTTT",
                    DescriptionOverride = "A very powerful modified light machine gun used by Void Division soldiers.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/VoidDivLightMachineGunView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        var firearm = Instance.GetComponent<FirearmBehaviour>();
                        firearm.Automatic = true;
                        firearm.AutomaticFireInterval = 0.055f;
                        firearm.InitialInaccuracy = 0.008f;
                        firearm.barrelPosition = new Vector3(0.8f, 0.08f);

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/VoidDivLightMachineGun.png");
                        Instance.FixColliders();

                        Cartridge customCartridge = ModAPI.FindCartridge("7.62x51mm");
                        customCartridge.name = "FS007 cartridge";
                        customCartridge.Damage *= 25f;
                        customCartridge.PenetrationRandomAngleMultiplier *= 0f;
                        customCartridge.StartSpeed *= 1.5f;
                        customCartridge.ImpactForce = 0.5f;
                        customCartridge.Recoil *= 0.3f;
                        firearm.Cartridge = customCartridge;
                    }
                }
            );
            //BR-22
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Shotgun"),
                    NameOverride = "BR-22 -OF",
                    NameToOrderByOverride = "TTTTTa",
                    DescriptionOverride = "A tactical shotgun modified to be fully automatic. Packs a huge punch. Used by Void Division Soldiers.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/VoidDivShotgunView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/VoidDivShotgun.png", 1f);
                        Instance.FixColliders();

                        var firearm = Instance.GetComponent<FirearmBehaviour>();
                        firearm.InitialInaccuracy = 0.1f;
                        firearm.BulletsPerShot = 8;
                        firearm.AutomaticFireInterval = 0.25f;
                        firearm.Automatic = true;

                        Cartridge customCartridge = ModAPI.FindCartridge("7.62x51mm");
                        customCartridge.name = "Automatic Shotgun cartridge";
                        customCartridge.Damage *= 7f;
                        customCartridge.Recoil *= 0.5f;
                        customCartridge.PenetrationRandomAngleMultiplier *= 0f;
                        customCartridge.StartSpeed *= 1f;
                        customCartridge.ImpactForce = 0.7f;
                        firearm.Cartridge = customCartridge;
                    }
                }
            );
            //T4-CHY-0N
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Stormbaan"),
                    NameOverride = "T4-CHY-0N -OF",
                    NameToOrderByOverride = "ZZZZZZZZZZ",
                    DescriptionOverride = "An incredibly heavily modified Transvorium Blaster, capable of firing a constant stream of electricity. Stupidly impractical, but looks cool as fuck. Created by none other than Sebastian Manus himself.",
                    CategoryOverride = ModAPI.FindCategory("? -OF"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/VoidDivLightMachineGunView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/T4-CHY-0N.png");
                        Instance.FixColliders();
                    }
                }
            );
            //Void Division Grenade
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Pink Explosive"),
                    NameOverride = "Void Division Grenade -OF",
                    NameToOrderByOverride = "TTTTTTT",
                    DescriptionOverride = "A handheld thowable explosive that uses Contraurium to generate a powerful explosion. Used by the Void Division.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/VoidDivGrenadeView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/VoidDivGrenade.png");
                        Instance.FixColliders();

                        Instance.GetComponent<ExplosiveBehaviour>().Range = 3.5f;
                        Instance.GetComponent<ExplosiveBehaviour>().Delay = 4f;

                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.1f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.1f;
                        Instance.GetComponent<PhysicalBehaviour>().rigidbody.mass = 0.1f;
                        AudioSource sound = Instance.AddComponent<AudioSource>();
                        sound.minDistance = 1;
                        sound.maxDistance = 1;
                        int index = 0;
                        AudioClip[] AudioData = new AudioClip[]
                        {
                            ModAPI.LoadSound("grenadebeep.wav"),
                        };
                        Instance.AddComponent<UseEventTrigger>().Action = () =>
                        {
                            sound.clip = AudioData[index];
                            sound.Play();
                            index++;
                            if (index == AudioData.Length) index = 0;
                        };
                    }
                }
            );
            //Void Division Pistol
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Glock 17"),
                    NameOverride = "Void Division Pistol -OF",
                    NameToOrderByOverride = "TTTT",
                    DescriptionOverride = "A handgun used by the Void Division.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/VoidDivPistolView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/VoidDivPistol.png", 1f);
                        Instance.FixColliders();
                    }
                }
            );
            //Void Division Knife
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Knife"),
                    NameOverride = "Void Division Knife -OF",
                    NameToOrderByOverride = "TTT",
                    DescriptionOverride = "A combat knife used by the Void Division",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/VoidDivKnifeView.png", 5f),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/VoidDivKnife.png", 1f);
                        Instance.FixColliders();
                    }
                }
            );
            //Void Division Combat Radio
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Void Division Combat Radio -OF",
                    NameToOrderByOverride = "TTTTTTTT",
                    DescriptionOverride = "A small handheld device that allows people to communicate through radio waves used by the Void Division.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/VoidDivCombatRadioView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/VoidDivCombatRadio.png");
                        Instance.FixColliders();

                        AudioSource sound = Instance.AddComponent<AudioSource>();
                        sound.minDistance = 1;
                        sound.maxDistance = 1;
                        int index = 0;
                        AudioClip[] AudioData = new AudioClip[]
                        {
                            ModAPI.LoadSound("Radio1.wav"),
                            ModAPI.LoadSound("Radio2.wav"),
                            ModAPI.LoadSound("Radio3.wav"),
                            ModAPI.LoadSound("Radio4.wav"),
                        };
                        Instance.AddComponent<UseEventTrigger>().Action = () =>
                        {
                            sound.clip = AudioData[index];
                            sound.Play();
                            index++;
                            if (index == AudioData.Length) index = 0;
                        };
                        Instance.GetComponent<GlowingHotMetalBehaviour>().enabled = false;
                    }
                }
            );
            //Electron Rifle
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Machine Blaster"),
                    NameOverride = "Ostrumian Electron Rifle -OF",
                    NameToOrderByOverride = "Ra",
                    DescriptionOverride = "A powerful automatic energy weapon which fires short bursts of electrons.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/EnergyRifleView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/EnergyRifle.png");
                        Instance.FixColliders();
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.45f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.45f;

                        var blaster = Instance.GetComponent<BlasterBehaviour>();
                        blaster.Automatic = true;
                        blaster.Recoil = 0.35f;
                        blaster.barrelPosition = new Vector2(0.4f, 0.085f);
                    }
                }
            );
            //Muon Rifle
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Blaster Rifle"),
                    NameOverride = "Ostrumian Muon Rifle -OF",
                    NameToOrderByOverride = "Raa",
                    DescriptionOverride = "A powerful semi-automatic energy weapon which fires short bursts of muons.",
                    CategoryOverride = ModAPI.FindCategory("? -OF"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/EnergyRifleView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/MuonRifle.png");
                        Instance.FixColliders();
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.55f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.55f;

                        var blaster = Instance.GetComponent<BlasterBehaviour>();
                        blaster.Automatic = false;
                        blaster.Recoil = 0.55f;
                        blaster.barrelPosition = new Vector2(0.5f, 0.07f);
                    }
                }
            );
            //Ion Cannon
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Ion Cannon"),
                    NameOverride = "Ostrumian Ion Cannon -OF",
                    NameToOrderByOverride = "Rb",
                    DescriptionOverride = "A powerful semi-automatic energy weapon that fires spheres of ionized plasma.",
                    CategoryOverride = ModAPI.FindCategory("OF - Tools"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/IonCannonView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/IonCannon.png");
                        Instance.FixColliders();
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.65f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.65f;
                    }
                }
            );
            //S-DEVICE
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Accelerator"),
                    NameOverride = "S-DEVICE -OF",
                    DescriptionOverride = "A weapon that charges up to fire balls of energy. Based off of a weapon I saw in a dream.",
                    CategoryOverride = ModAPI.FindCategory("? -OF"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/IonCannonView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/S-DEVICE.png");
                        Instance.GetComponent<PhysicalBehaviour>().InitialMass = 0.65f;
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.65f;
                    }
                }
            );
            //Dark-Matter Sphere
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Wheel"),
                    NameOverride = "Dark-Matter Sphere -OF",
                    NameToOrderByOverride = "Z",
                    DescriptionOverride = "A sphere of pure dark-matter. It appears to create darkness around itself.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/DarkMatterView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/DarkMatterSphere.png");
                        Instance.AddComponent<DarkMatter>();
                        Instance.AddComponent<Freeze>();

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Soft");
                    }
                }
            );
            //Black Hole
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Wheel"),
                    NameOverride = "Black Hole -OF",
                    NameToOrderByOverride = "ZZZZZZZZZZZZZZZZZZZZZZZZZZBALLSLMAOPENISHEHE",
                    DescriptionOverride = "A black hole for scenes",
                    CategoryOverride = ModAPI.FindCategory("? -OF"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/DarkMatterView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/DarkMatterSphere.png");
                        Instance.AddComponent<BlackHole1>();
                        Instance.AddComponent<BlackHole2>();
                        Instance.AddComponent<Freeze>();

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.Properties = ModAPI.FindPhysicalProperties("Soft");
                    }
                }
            );
            //Fusion Cannon
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Cube"),
                    NameOverride = "Ostrumian Fusion Cannon -OF",
                    DescriptionOverride = "Creates incredibly high pressures and heat within a chamber to make particles of Hydrogen, Deuterium and Tritium undergo fusion, and blasts them out the front all at once. This is done through the use of Gravitasium, Quarium, Duplium and Ferrumitanium.",
                    CategoryOverride = ModAPI.FindCategory("OF - Technology"),
                    ThumbnailOverride = ModAPI.LoadSprite("view/FusionCannonView.png"),
                    AfterSpawn = (Instance) =>
                    {
                        SunPower sun = Instance.AddComponent<SunPower>();

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprite/FusionCannon.png");
                        Instance.FixColliders();
                        AudioSource AS = Instance.AddComponent<AudioSource>();
                        AS.minDistance = 1;
                        AS.maxDistance = 100;
                        int index = 0; AudioClip[] AudioData = new AudioClip[]
                        {
                            ModAPI.LoadSound("fusioncannonshot.wav"),
                        };
                        Instance.AddComponent<UseEventTrigger>().Action = () =>
                        {
                            AS.clip = AudioData[index]; AS.Play(); index++; if (index == AudioData.Length) index = 0;
                        };
                    }
                }
            );
        }
    }

    public class EMPDeath : MonoBehaviour
    {
        Liquid blood1;
        LimbBehaviour Limb;
        PersonBehaviour Person;

        public void OnEMPHit()
        {
            Limb = gameObject.GetComponent<LimbBehaviour>();
            Person = gameObject.GetComponent<PersonBehaviour>();
            Limb.Health = 0f;
        }
    }

    public class ParticleOnExplode : MonoBehaviour
    {
        ExplosiveBehaviour ex;
        public string Disintegration;

        void Start()
        {
            ex = GetComponent<ExplosiveBehaviour>(); ex.OnExplode.AddListener(Explode);
        }

        void Explode()
        {
            ModAPI.CreateParticleEffect("HugeZap", transform.position);
            ModAPI.CreateParticleEffect("HugeZap", transform.position);
            ModAPI.CreateParticleEffect("HugeZap", transform.position);
            ModAPI.CreateParticleEffect("HugeZap", transform.position);
            ModAPI.CreateParticleEffect("HugeZap", transform.position);
            ModAPI.CreateParticleEffect("Vapor", transform.position);
            ModAPI.CreateParticleEffect("Vapor", transform.position);
            ModAPI.CreateParticleEffect("Vapor", transform.position);
            ModAPI.CreateParticleEffect("Vapor", transform.position);
            ModAPI.CreateParticleEffect("Vapor", transform.position);
        }
    }
}